@@ -1,5 +1,5 @@
 /*
- * $Id: action.c,v 1.151 2004/01/21 20:03:15 tal197 Exp $
+ * $Id: action.c,v 1.154 2004/07/30 17:08:38 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -841,10 +841,10 @@ static GUIside *start_action(GtkWidget *abox, ActionChild *func, gpointer data,
 	g_signal_connect(abox, "abort_operation",
 			 G_CALLBACK(abort_operation), gui_side);
 
-	gui_side->input_tag = gtk_input_add_full(gui_side->from_child,
+	gui_side->input_tag = gdk_input_add_full(gui_side->from_child,
 						GDK_INPUT_READ,
 						message_from_child,
-						NULL, gui_side, NULL);
+						gui_side, NULL);
 
 	return gui_side;
 }
@@ -1325,7 +1325,7 @@ static void do_copy2(const char *path, const char *dest)
 				  _("?Copy %s as %s?"), path, dest_path))
 			return;
 	}
-	else
+	else if (!o_brief || S_ISDIR(info.st_mode))
 		printf_send(_("'Copying %s as %s\n"), path, dest_path);
 
 	if (S_ISDIR(info.st_mode))
@@ -1483,7 +1483,7 @@ static void do_move2(const char *path, const char *dest)
 				  _("?Move %s as %s?"), path, dest_path))
 			return;
 	}
-	else
+	else if (!o_brief)
 		printf_send(_("'Moving %s as %s\n"), path, dest_path);
 
 	argv[2] = path;
@@ -2130,6 +2130,9 @@ void action_copy(GList *paths, const char *dest, const char *leaf, int quiet)
 		   _("Newer"),
 		   _("Only over-write if source is newer than destination."),
 		   'W', o_action_newer.int_value);
+	abox_add_flag(ABOX(abox),
+		_("Brief"), _("Only log directories as they are copied"),
+		'B', o_action_brief.int_value);
 
 	number_of_windows++;
 	gtk_widget_show(abox);
@@ -2163,6 +2166,9 @@ void action_move(GList *paths, const char *dest, const char *leaf, int quiet)
 		   _("Newer"),
 		   _("Only over-write if source is newer than destination."),
 		   'W', o_action_newer.int_value);
+	abox_add_flag(ABOX(abox),
+		_("Brief"), _("Don't log each file as it is moved"),
+		'B', o_action_brief.int_value);
 	number_of_windows++;
 	gtk_widget_show(abox);
 }
