@@ -1,5 +1,5 @@
 /*
- * $Id: appmenu.c,v 1.30 2003/01/19 19:29:03 tal197 Exp $
+ * $Id: appmenu.c,v 1.41 2004/01/06 12:56:16 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -21,6 +21,8 @@
 
 /* appmenu.c - handles application-specific menus read from XMLwrapper.xml */
 
+/* XXX: This handles all File menu extensions. Needs renaming! */
+
 #include "config.h"
 
 #include <gtk/gtk.h>
@@ -33,6 +35,7 @@
 
 #include "global.h"
 
+#include "choices.h"
 #include "fscache.h"
 #include "gui_support.h"
 #include "support.h"
@@ -43,10 +46,17 @@
 #include "type.h"
 #include "appinfo.h"
 #include "xml.h"
+#include "run.h"
+#include "diritem.h"
+#include "action.h"
+#include "options.h"
 
 /* Static prototypes */
 static void apprun_menu(GtkWidget *item, gpointer data);
 static GtkWidget *create_menu_item(xmlNode *node);
+static void show_app_help(GtkWidget *item, gpointer data);
+static void build_app_menu(const char *app_dir, DirItem *app_item);
+static void mnt_eject(GtkWidget *item, gpointer data);
 
 /* There can only be one menu open at a time... we store: */
 static GtkWidget *current_menu = NULL;	/* The GtkMenu */
@@ -54,7 +64,6 @@ static guchar *current_app_path = NULL;	/* The path of the application */
 static GList *current_items = NULL;	/* The GtkMenuItems we added directly
 					 * to it --- not submenu items.
 					 */
-
 /****************************************************************
  *			EXTERNAL INTERFACE			*
  ****************************************************************/
@@ -81,62 +90,53 @@ void appmenu_remove(void)
  * This function modifies the menu stored in "menu".
  * 'app_dir' is the pathname of the application directory, and 'item'
  * is the corresponding DirItem.
+ * Returns number of entries added.
  * Call appmenu_remove() to undo the effect.
  */
-void appmenu_add(const gchar *app_dir, DirItem *item, GtkWidget *menu)
+int appmenu_add(const gchar *app_dir, DirItem *app_item, GtkWidget *menu)
 {
-	XMLwrapper	*ai = NULL;
-	xmlNode	*node;
 	GList	*next;
 	GtkWidget *sep;
+	int nadded = 0;
 
-	g_return_if_fail(menu != NULL);
+	g_return_val_if_fail(menu != NULL, 0);
 
 	/* Should have called appmenu_remove() already... */
-	g_return_if_fail(current_menu == NULL);
+	g_return_val_if_fail(current_menu == NULL, 0);
+	g_return_val_if_fail(current_items == NULL, 0);
 
-	ai = appinfo_get(app_dir, item);
-	if (!ai)
-		return;	/* No appmenu (or invalid XML) */
+	build_app_menu(app_dir, app_item);
 
-	/* OK, we've got a valid info file and parsed it.
-	 * Now build the GtkMenuItem widgets and add them to the menu
-	 * (if there are any).
-	 */
-
-	node = xml_get_section(ai, NULL, "AppMenu");
-	if (!node)
-		goto out;
-
-	g_return_if_fail(current_items == NULL);
-
-	/* Add the menu entries */
-	for (node = node->xmlChildrenNode; node; node = node->next)
+	if (app_item->flags & ITEM_FLAG_MOUNT_POINT)
 	{
 		GtkWidget *item;
+		item = gtk_menu_item_new_with_label(_("Eject"));
+		gtk_widget_show(item);
+		current_items = g_list_prepend(current_items, item);
+		g_signal_connect(item, "activate", G_CALLBACK(mnt_eject), NULL);
+	}
 
-		item = create_menu_item(node);
-
-		if (item)
-			current_items = g_list_prepend(current_items, item);
+	if (current_items)
+	{
+		sep = gtk_menu_item_new();
+		current_items = g_list_prepend(current_items, sep);
+		gtk_widget_show(sep);
 	}
-		
-	sep = gtk_menu_item_new();
-	current_items = g_list_prepend(current_items, sep);
-	gtk_widget_show(sep);
 
 	for (next = current_items; next; next = next->next)
+	{
 		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu),
 					GTK_WIDGET(next->data));
+		nadded++;
+	}
 
 	current_menu = menu;
 	current_app_path = g_strdup(app_dir);
 
-out:
-	if (ai)
-		g_object_unref(ai);
+	return nadded;
 }
 
+
 /****************************************************************
  *                      INTERNAL FUNCTIONS                      *
  ****************************************************************/
@@ -231,6 +231,18 @@ static GtkWidget *create_menu_item(xmlNode *node)
 	return item;
 }
 
+/* Send to current_app_path (though not actually an app) */
+static void send_to(GtkWidget *item, const char *app)
+{
+	GList *file_list;
+
+	g_return_if_fail(current_app_path != NULL);
+
+	file_list = g_list_prepend(NULL, current_app_path);
+	run_with_files(app, file_list);
+	g_list_free(file_list);
+}
+
 /* Function called to execute an AppMenu item */
 static void apprun_menu(GtkWidget *item, gpointer data)
 {
@@ -249,3 +261,130 @@ static void apprun_menu(GtkWidget *item, gpointer data)
 
 	g_free(argv[0]);
 }
+
+static void show_app_help(GtkWidget *item, gpointer data)
+{
+	g_return_if_fail(current_app_path != NULL);
+
+	show_help_files(current_app_path);
+}
+
+static void mnt_eject(GtkWidget *item, gpointer data)
+{
+	GList *dirs;
+
+	g_return_if_fail(current_app_path != NULL);
+	dirs = g_list_prepend(NULL, current_app_path);
+	action_eject(dirs);
+	g_list_free(dirs);
+}
+
+static void customise_type(GtkWidget *item, MIME_type *type)
+{
+	char *leaf;
+	char *path;
+
+	leaf = g_strconcat(".", type->media_type, "_", type->subtype, NULL);
+	path = choices_find_path_save(leaf, "SendTo", TRUE);
+	g_free(leaf);
+
+	mkdir(path, 0755);
+	filer_opendir(path, NULL, NULL);
+	g_free(path);
+
+	info_message(_("Symlink any programs you want into this directory. "
+			"They will appear in the menu for all items of this "
+			"type (%s/%s)."), type->media_type, type->subtype);
+}
+
+static void build_menu_for_type(MIME_type *type)
+{
+	GPtrArray *names;
+	char *path;
+	int i;
+	char *leaf;
+	GtkWidget *item;
+	DirItem *ditem;
+
+	leaf = g_strconcat(".", type->media_type, "_", type->subtype, NULL);
+	path = choices_find_path_load(leaf, "SendTo");
+
+	if (!path)
+		goto out;
+
+	names = list_dir(path);
+
+	ditem = diritem_new("");
+
+	for (i = 0; i < names->len; i++)
+	{
+		char	*leaf = names->pdata[i];
+		char	*full_path;
+
+		full_path = g_build_filename(path, leaf, NULL);
+		diritem_restat(full_path, ditem, NULL);
+		
+		item = make_send_to_item(ditem, leaf, MIS_SMALL);
+		current_items = g_list_prepend(current_items, item);
+		gtk_widget_show(item);
+		g_signal_connect_data(item, "activate", G_CALLBACK(send_to),
+				full_path, (GClosureNotify) g_free, 0);
+	}
+
+	g_ptr_array_free(names, TRUE);
+
+	g_free(path);
+
+out:
+	item = gtk_menu_item_new_with_label(_("Customise Menu..."));
+	current_items = g_list_prepend(current_items, item);
+	g_signal_connect(item, "activate", G_CALLBACK(customise_type), type);
+	gtk_widget_show(item);
+
+	g_free(leaf);
+}
+
+/* Adds to current_items */
+static void build_app_menu(const char *app_dir, DirItem *app_item)
+{
+	XMLwrapper	*ai = NULL;
+	xmlNode	*node;
+	GtkWidget *item;
+
+	ai = appinfo_get(app_dir, app_item);
+	if (ai)
+	{
+		node = xml_get_section(ai, NULL, "AppMenu");
+		if (node)
+			node = node->xmlChildrenNode;
+	}
+	else
+	{
+		if (app_item->flags & ITEM_FLAG_APPDIR)
+			node = NULL;
+		else
+		{
+			/* Not an application AND no AppInfo */
+			build_menu_for_type(app_item->mime_type);
+			return;
+		}
+	}
+
+	/* Add the menu entries */
+	for (; node; node = node->next)
+	{
+		item = create_menu_item(node);
+
+		if (item)
+			current_items = g_list_prepend(current_items, item);
+	}
+		
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_HELP, NULL);
+	gtk_widget_show(item);
+	current_items = g_list_prepend(current_items, item);
+	g_signal_connect(item, "activate", G_CALLBACK(show_app_help), NULL);
+	gtk_label_set_text(GTK_LABEL(GTK_BIN(item)->child), _("Help"));
+
+	if (ai)
+		g_object_unref(ai);
+}
