@@ -1,5 +1,5 @@
 /*
- * $Id: dir.c,v 1.92.2.1 2003/07/23 15:45:09 tal197 Exp $
+ * $Id: dir.c,v 1.95 2004/07/30 17:08:39 tal197 Exp $
  *
  * Copyright (C) 2003, the ROX-Filer team.
  *
@@ -78,6 +78,9 @@ gboolean dnotify_wakeup_flag = FALSE;
 static int dnotify_last_fd = -1;
 #endif
 
+/* For debugging. Can't detach when this is non-zero. */
+static int in_callback = 0;
+
 GFSCache *dir_cache = NULL;
 
 /* Static prototypes */
@@ -196,6 +199,7 @@ void dir_detach(Directory *dir, DirCallback callback, gpointer data)
 
 	g_return_if_fail(dir != NULL);
 	g_return_if_fail(callback != NULL);
+	g_return_if_fail(in_callback == 0);
 
 	for (list = dir->users; list; list = list->next)
 	{
@@ -332,6 +336,8 @@ static void dir_set_scanning(Directory *dir, gboolean scanning)
 	if (scanning == dir->scanning)
 		return;
 
+	in_callback++;
+
 	dir->scanning = scanning;
 
 	for (next = dir->users; next; next = next->next)
@@ -342,6 +348,8 @@ static void dir_set_scanning(Directory *dir, gboolean scanning)
 				scanning ? DIR_START_SCAN : DIR_END_SCAN,
 				NULL, user->data);
 	}
+
+	in_callback--;
 }
 
 /* Notify everyone that the error status of the directory has changed */
@@ -349,12 +357,16 @@ static void dir_error_changed(Directory *dir)
 {
 	GList	*next;
 
+	in_callback++;
+
 	for (next = dir->users; next; next = next->next)
 	{
 		DirUser *user = (DirUser *) next->data;
 
 		user->callback(dir, DIR_ERROR_CHANGED, NULL, user->data);
 	}
+
+	in_callback--;
 }
 
 /* This is called in the background when there are items on the
@@ -392,7 +404,7 @@ static gboolean recheck_callback(gpointer data)
 	
 	dir->have_scanned = TRUE;
 	dir_set_scanning(dir, FALSE);
-	gtk_idle_remove(dir->idle_callback);
+	g_source_remove(dir->idle_callback);
 	dir->idle_callback = 0;
 
 	if (dir->needs_update)
@@ -411,6 +423,8 @@ void dir_merge_new(Directory *dir)
 	GPtrArray *gone = dir->gone_items;
 	GList	  *list;
 	guint	  i;
+	
+	in_callback++;
 
 	for (list = dir->users; list; list = list->next)
 	{
@@ -424,6 +438,8 @@ void dir_merge_new(Directory *dir)
 			user->callback(dir, DIR_REMOVE, gone, user->data);
 	}
 
+	in_callback--;
+
 	for (i = 0; i < new->len; i++)
 	{
 		DirItem *item = (DirItem *) new->pdata[i];
@@ -483,7 +499,7 @@ static void dir_rescan_soon(Directory *dir)
 {
 	if (dir->rescan_timeout != -1)
 		return;
-	dir->rescan_timeout = gtk_timeout_add(500, rescan_soon_timeout, dir);
+	dir->rescan_timeout = g_timeout_add(500, rescan_soon_timeout, dir);
 }
 #endif
 
@@ -509,12 +525,16 @@ static void notify_deleted(Directory *dir, GPtrArray *deleted)
 	if (!deleted->len)
 		return;
 
+	in_callback++;
+
 	for (next = dir->users; next; next = next->next)
 	{
 		DirUser *user = (DirUser *) next->data;
 
 		user->callback(dir, DIR_REMOVE, deleted, user->data);
 	}
+
+	in_callback--;
 }
 
 static void mark_unused(gpointer key, gpointer value, gpointer data)
@@ -596,7 +616,7 @@ static void delayed_notify(Directory *dir)
 	if (dir->notify_active)
 		return;
 	g_object_ref(dir);
-	gtk_timeout_add(1500, notify_timeout, dir);
+	g_timeout_add(1500, notify_timeout, dir);
 	dir->notify_active = TRUE;
 }
 
@@ -712,7 +732,7 @@ static void set_idle_callback(Directory *dir)
 		if (dir->idle_callback)
 			return;
 		time(&diritem_recent_time);
-		dir->idle_callback = gtk_idle_add(recheck_callback, dir);
+		dir->idle_callback = g_idle_add(recheck_callback, dir);
 		/* Do the first call now (will remove the callback itself) */
 		recheck_callback(dir);
 	}
@@ -721,7 +741,7 @@ static void set_idle_callback(Directory *dir)
 		dir_set_scanning(dir, FALSE);
 		if (dir->idle_callback)
 		{
-			gtk_idle_remove(dir->idle_callback);
+			g_source_remove(dir->idle_callback);
 			dir->idle_callback = 0;
 		}
 	}
@@ -742,6 +762,8 @@ static void dir_force_update_item(Directory *dir, const gchar *leaf)
 
 	g_ptr_array_add(items, item);
 
+	in_callback++;
+
 	for (list = dir->users; list; list = list->next)
 	{
 		DirUser *user = (DirUser *) list->data;
@@ -749,6 +771,8 @@ static void dir_force_update_item(Directory *dir, const gchar *leaf)
 		user->callback(dir, DIR_UPDATE, items, user->data);
 	}
 
+	in_callback--;
+
 out:
 	g_ptr_array_free(items, TRUE);
 }
@@ -801,7 +825,7 @@ static void dir_finialize(GObject *object)
 	free_recheck_list(dir);
 	set_idle_callback(dir);
 	if (dir->rescan_timeout != -1)
-		gtk_timeout_remove(dir->rescan_timeout);
+		g_source_remove(dir->rescan_timeout);
 
 	dir_merge_new(dir);	/* Ensures new, up and gone are empty */
 
