@@ -1,6 +1,5 @@
 /*
- * $Id: dir.h,v 1.27 2002/01/21 15:44:32 tal197 Exp $
- *
+ * ROX-Filer, filer for the ROX desktop project
  * Thomas Leonard, <tal197@users.sourceforge.net>
  */
 
@@ -11,12 +10,22 @@
 #include <sys/types.h>
 #include <dirent.h>
 
+#include <signal.h>
+#include <fcntl.h>
+
+/* Check for dnotify support */
+#if defined(DN_MULTISHOT) && defined(SIGRTMIN)
+# define USE_DNOTIFY
+extern gboolean dnotify_wakeup_flag;
+#endif
+
 typedef enum {
 	DIR_START_SCAN,	/* Set 'scanning' indicator */
 	DIR_END_SCAN,	/* Clear 'scanning' indicator */
 	DIR_ADD,	/* Add the listed items to the display */
 	DIR_REMOVE,	/* Remove listed items from display */
 	DIR_UPDATE,	/* Redraw these items */
+	DIR_ERROR_CHANGED,	/* Check dir->error */
 } DirAction;
 
 typedef struct _DirUser DirUser;
@@ -33,13 +42,22 @@ struct _DirUser
 	gpointer	data;
 };
 
+typedef struct _DirectoryClass DirectoryClass;
+
+struct _DirectoryClass {
+	GObjectClass parent;
+};
+
 struct _Directory
 {
+	GObject object;
+
 	char	*pathname;	/* Internal use only */
-	int	ref;
 	GList	*users;		/* Functions to call on update */
 	char	*error;		/* NULL => no error */
 
+	struct stat	stat_info;	/* Internal use */
+
 	gboolean	notify_active;	/* Notify timeout is running */
 	gint		idle_callback;	/* Idle callback ID */
 
@@ -62,18 +80,24 @@ struct _Directory
 	 * and scanning is not in progress, a rescan is triggered.
 	 */
 	gboolean	needs_update;
+
+	gint		rescan_timeout;	/* See dir_rescan_soon() */
+
+#ifdef USE_DNOTIFY
+	int		dnotify_fd;	/* -1 if not watching */
+#endif
 };
 
 void dir_init(void);
 void dir_attach(Directory *dir, DirCallback callback, gpointer data);
 void dir_detach(Directory *dir, DirCallback callback, gpointer data);
 void dir_update(Directory *dir, gchar *pathname);
-int dir_item_cmp(const void *a, const void *b);
-void refresh_dirs(char *path);
-void dir_check_this(guchar *path);
-DirItem *dir_update_item(Directory *dir, gchar *leafname);
-void dir_rescan(Directory *dir, guchar *pathname);
+void refresh_dirs(const char *path);
+void dir_check_this(const guchar *path);
+DirItem *dir_update_item(Directory *dir, const gchar *leafname);
 void dir_merge_new(Directory *dir);
-void dir_force_update_path(gchar *path);
+void dir_force_update_path(const gchar *path);
+void dnotify_wakeup(void);
+void dir_drop_all_dnotifies(void);
 
 #endif /* _DIR_H */
