@@ -1,5 +1,5 @@
 /*
- * $Id: filer.c,v 1.344 2004/04/13 09:04:04 tal197 Exp $
+ * $Id: filer.c,v 1.348 2004/07/30 17:08:39 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -236,6 +236,8 @@ static gboolean if_deleted(gpointer item, gpointer removed)
 	return FALSE;
 }
 
+#define DECOR_BORDER 32
+
 /* Resize the filer window to w x h pixels, plus border (not clamped).
  * If triggered by a key event, warp the pointer (for SloppyFocus users).
  */
@@ -258,20 +260,34 @@ void filer_window_set_size(FilerWindow *filer_window, int w, int h)
 
 	if (GTK_WIDGET_VISIBLE(window))
 	{
-		gint x, y;
+		gint x, y, m;
 		GtkRequisition	*req = &window->requisition;
 		GdkWindow *gdk_window = window->window;
 
 		w = MAX(req->width, w);
 		h = MAX(req->height, h);
+		gdk_window_get_pointer(NULL, &x, &y, NULL);
+		m = gdk_screen_get_monitor_at_point(gdk_screen_get_default(),
+				x, y);
 		gdk_window_get_position(gdk_window, &x, &y);
 
-		if (x + w > monitor_width || y + h > monitor_height)
+		if (x + w + DECOR_BORDER >
+				monitor_geom[m].x + monitor_geom[m].width ||
+				y + h + DECOR_BORDER >
+				monitor_geom[m].y + monitor_geom[m].height)
 		{
-			if (x + w > monitor_width)
-				x = monitor_width - w - 4;
-			if (y + h > monitor_height)
-				y = monitor_height - h - 4;
+			if (x + w + DECOR_BORDER >
+				monitor_geom[m].x + monitor_geom[m].width)
+			{
+				x = monitor_geom[m].x + monitor_geom[m].width -
+					w - 4 - DECOR_BORDER;
+			}
+			if (y + h + DECOR_BORDER >
+				monitor_geom[m].y + monitor_geom[m].height)
+			{
+				y = monitor_geom[m].y + monitor_geom[m].height -
+					h - 4 - DECOR_BORDER;
+			}
 			gdk_window_move_resize(gdk_window, x, y, w, h);
 		}
 		else
@@ -302,7 +318,7 @@ static gint open_filer_window(FilerWindow *filer_window)
 
 	if (filer_window->open_timeout)
 	{
-		gtk_timeout_remove(filer_window->open_timeout);
+		g_source_remove(filer_window->open_timeout);
 		filer_window->open_timeout = 0;
 	}
 
@@ -465,13 +481,13 @@ static void filer_window_destroyed(GtkWidget *widget, FilerWindow *filer_window)
 
 	if (filer_window->open_timeout)
 	{
-		gtk_timeout_remove(filer_window->open_timeout);
+		g_source_remove(filer_window->open_timeout);
 		filer_window->open_timeout = 0;
 	}
 
 	if (filer_window->auto_scroll != -1)
 	{
-		gtk_timeout_remove(filer_window->auto_scroll);
+		g_source_remove(filer_window->auto_scroll);
 		filer_window->auto_scroll = -1;
 	}
 
@@ -1147,8 +1163,7 @@ void filer_change_to(FilerWindow *filer_window,
 		view_autosize(filer_window->view);
 
 	if (filer_window->mini_type == MINI_PATH)
-		gtk_idle_add((GtkFunction) minibuffer_show_cb,
-				filer_window);
+		g_idle_add((GSourceFunc) minibuffer_show_cb, filer_window);
 }
 
 /* Returns a list containing the full (sym) pathname of every selected item.
@@ -1200,6 +1215,7 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 	DetailsType     dtype;
 	SortType	s_type;
 	GtkSortType	s_order;
+	Settings	*dir_settings = NULL;
 
 	/* Get the real pathname of the directory and copy it */
 	real_path = pathdup(path);
@@ -1293,27 +1309,67 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 		filer_window->view_type = o_filer_view_type.int_value;
 	}
 
+	dir_settings = (Settings *) g_hash_table_lookup(settings_table,
+					      filer_window->sym_path);
+	if (dir_settings)
+	{
+		/* Override the current defaults with the per-directory
+		 * user settings.
+		 */
+		if (dir_settings->flags & SET_HIDDEN)
+			filer_window->show_hidden = dir_settings->show_hidden;
+
+		if (dir_settings->flags & SET_STYLE)
+			filer_window->display_style =
+				dir_settings->display_style;
+
+		if (dir_settings->flags & SET_DETAILS)
+			filer_window->details_type = dir_settings->details_type;
+
+		if (dir_settings->flags & SET_SORT)
+		{
+			s_type = dir_settings->sort_type;
+			s_order = dir_settings->sort_order;
+		}
+
+		if (dir_settings->flags & SET_THUMBS)
+			filer_window->show_thumbs = dir_settings->show_thumbs;
+		
+		if (dir_settings->flags & SET_FILTER)
+			filer_set_filter(filer_window,
+					   dir_settings->filter_type,
+					   dir_settings->filter);
+	}
+
 	/* Add all the user-interface elements & realise */
 	filer_add_widgets(filer_window, wm_class);
 	if (src_win)
 		gtk_window_set_position(GTK_WINDOW(filer_window->window),
 					GTK_WIN_POS_MOUSE);
 
+	if (dir_settings)
+	{
+		if(dir_settings->flags & SET_POSITION) 
+			gtk_window_move(GTK_WINDOW(filer_window->window),
+					    dir_settings->x, dir_settings->y);
+		if(dir_settings->flags & SET_SIZE) 
+			filer_window_set_size(filer_window,
+					      dir_settings->width,
+					      dir_settings->height);
+	}
+
 	/* Connect to all the signal handlers */
 	filer_add_signals(filer_window);
 
 	display_set_layout(filer_window, dstyle, dtype, TRUE);
 	display_set_sort_type(filer_window, s_type, s_order);
 
-	/* Do we have saved settings? */
-	check_settings(filer_window);
-
 	/* Open the window after a timeout, or when scanning stops.
 	 * Do this before attaching, because attach() might tell us to
 	 * stop scanning (if a scan isn't needed).
 	 */
-	filer_window->open_timeout = gtk_timeout_add(500,
-					  (GtkFunction) open_filer_window,
+	filer_window->open_timeout = g_timeout_add(500,
+					  (GSourceFunc) open_filer_window,
 					  filer_window);
 
 	/* The view is created empty and then attach() is called, which
@@ -1570,7 +1626,7 @@ static void set_scanning_display(FilerWindow *filer_window, gboolean scanning)
 	if (scanning)
 		filer_set_title(filer_window);
 	else
-		gtk_timeout_add(300, (GtkFunction) clear_scanning_display,
+		g_timeout_add(300, (GSourceFunc) clear_scanning_display,
 				filer_window);
 }
 
@@ -1967,7 +2023,7 @@ static void filer_next_thumb(GObject *window, const gchar *path)
 	if (path)
 		dir_force_update_path(path);
 
-	gtk_idle_add((GtkFunction) filer_next_thumb_real, window);
+	g_idle_add((GSourceFunc) filer_next_thumb_real, window);
 }
 
 static void start_thumb_scanning(FilerWindow *filer_window)
@@ -2588,8 +2644,8 @@ void filer_set_autoscroll(FilerWindow *filer_window, gboolean auto_scroll)
 		if (filer_window->auto_scroll != -1)
 			return;		/* Already on! */
 
-		filer_window->auto_scroll = gtk_timeout_add(50,
-					(GtkFunction) as_timeout,
+		filer_window->auto_scroll = g_timeout_add(50,
+					(GSourceFunc) as_timeout,
 					filer_window);
 	}
 	else
@@ -2597,7 +2653,7 @@ void filer_set_autoscroll(FilerWindow *filer_window, gboolean auto_scroll)
 		if (filer_window->auto_scroll == -1)
 			return;		/* Already off! */
 
-		gtk_timeout_remove(filer_window->auto_scroll);
+		g_source_remove(filer_window->auto_scroll);
 		filer_window->auto_scroll = -1;
 	}
 }
@@ -2970,7 +3026,6 @@ static void check_settings(FilerWindow *filer_window)
 			if(set->flags & SET_DETAILS) {
 				details=set->details_type;
 
-				/* This next causes a warning */
 				filer_set_view_type(filer_window,
 						    set->view_type);
 			}
