@@ -1,8 +1,8 @@
 /*
- * $Id: fscache.c,v 1.19 2002/01/22 16:55:52 tal197 Exp $
+ * $Id: fscache.c,v 1.28 2004/05/22 14:46:56 tal197 Exp $
  *
  * FSCache - a glib-style object for caching files
- * Copyright (C) 2002, the ROX-Filer team.
+ * Copyright (C) 2003, the ROX-Filer team.
  *
  * A cache object holds stat details about files in a hash table, along
  * with user-specified data. When you want to read in a file try to
@@ -34,8 +34,37 @@
 
 #include "fscache.h"
 
+typedef struct _GFSCacheKey GFSCacheKey;
+typedef struct _GFSCacheData GFSCacheData;
+
+struct _GFSCache
+{
+	GHashTable	*inode_to_stats;
+	GFSLoadFunc	load;
+	GFSUpdateFunc	update;
+	gpointer	user_data;
+};
+
+struct _GFSCacheKey
+{
+	dev_t		device;
+	ino_t		inode;
+};
+
+struct _GFSCacheData
+{
+	GObject		*data;		/* The object from the file */
+	time_t		last_lookup;
+
+	/* Details of the file last time we checked it */
+	time_t		m_time, c_time;
+	off_t		length;
+	mode_t		mode;
+};
+
 #define UPTODATE(data, info)				\
 		(data->m_time == info.st_mtime		\
+		 && data->c_time == info.st_ctime	\
 		 && data->length == info.st_size	\
 		 && data->mode == info.st_mode)		\
 
@@ -47,7 +76,7 @@ static gint cmp_stats(gconstpointer a, gconstpointer b);
 static void destroy_hash_entry(gpointer key, gpointer data, gpointer user_data);
 static gboolean purge_hash_entry(gpointer key, gpointer data,
 				 gpointer user_data);
-static GFSCacheData *lookup_internal(GFSCache *cache, char *pathname,
+static GFSCacheData *lookup_internal(GFSCache *cache, const char *pathname,
 					FSCacheLookup lookup_type);
 
 
@@ -84,9 +113,6 @@ struct PurgeInfo
  * 'user_data' will be passed to all of the above functions.
  */
 GFSCache *g_fscache_new(GFSLoadFunc load,
-			GFSRefFunc ref,
-			GFSRefFunc unref,
-			GFSGetRefFunc getref,
 			GFSUpdateFunc update,
 			gpointer user_data)
 {
@@ -95,9 +121,6 @@ GFSCache *g_fscache_new(GFSLoadFunc load,
 	cache = g_new(GFSCache, 1);
 	cache->inode_to_stats = g_hash_table_new(hash_key, cmp_stats);
 	cache->load = load;
-	cache->ref = ref;
-	cache->unref = unref;
-	cache->getref = getref;
 	cache->update = update;
 	cache->user_data = user_data;
 
@@ -108,39 +131,21 @@ void g_fscache_destroy(GFSCache *cache)
 {
 	g_return_if_fail(cache != NULL);
 
-	g_hash_table_foreach(cache->inode_to_stats, destroy_hash_entry, cache);
+	g_hash_table_foreach(cache->inode_to_stats, destroy_hash_entry, NULL);
 	g_hash_table_destroy(cache->inode_to_stats);
 
 	g_free(cache);
 }
 
-/* Call the ref() user function for this object */
-void g_fscache_data_ref(GFSCache *cache, gpointer data)
-{
-	g_return_if_fail(cache != NULL);
-
-	if (cache->ref)
-		cache->ref(data, cache->user_data);
-}
-
-/* Call the unref() user function for this object */
-void g_fscache_data_unref(GFSCache *cache, gpointer data)
-{
-	g_return_if_fail(cache != NULL);
-
-	if (cache->unref)
-		cache->unref(data, cache->user_data);
-}
-
 /* Find the data for this file in the cache, loading it into
  * the cache if it isn't there already.
  *
- * Remember to g_fscache_data_unref() the returned value when
+ * Remember to g_object_unref() the returned value when
  * you're done with it.
  *
  * Returns NULL on failure.
  */
-gpointer g_fscache_lookup(GFSCache *cache, char *pathname)
+gpointer g_fscache_lookup(GFSCache *cache, const char *pathname)
 {
 	return g_fscache_lookup_full(cache, pathname,
 			FSCACHE_LOOKUP_CREATE, NULL);
@@ -153,7 +158,7 @@ gpointer g_fscache_lookup(GFSCache *cache, char *pathname)
  * start loading some data and then with update_details = FALSE when you
  * put in the loaded object.
  */
-void g_fscache_insert(GFSCache *cache, char *pathname, gpointer obj,
+void g_fscache_insert(GFSCache *cache, const char *pathname, gpointer obj,
 		      gboolean update_details)
 {
 	GFSCacheData	*data;
@@ -165,11 +170,11 @@ void g_fscache_insert(GFSCache *cache, char *pathname, gpointer obj,
 	if (!data)
 		return;
 
-	if (cache->unref)
-		cache->unref(data->data, cache->user_data);
+	if (obj)
+		g_object_ref(obj);
+	if (data->data)
+		g_object_unref(data->data);
 	data->data = obj;
-	if (cache->ref)
-		cache->ref(data->data, cache->user_data);
 }
 
 /* As g_fscache_lookup, but 'lookup_type' controls what happens if the data
@@ -177,8 +182,9 @@ void g_fscache_insert(GFSCache *cache, char *pathname, gpointer obj,
  * If found is not NULL, use it to indicate whether something is being
  * returned (a NULL return could indicate that the data is cached, but
  * the data is NULL).
+ * If returned value is not NULL, value is refd.
  */
-gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
+gpointer g_fscache_lookup_full(GFSCache *cache, const char *pathname,
 				FSCacheLookup lookup_type,
 				gboolean *found)
 {
@@ -198,8 +204,8 @@ gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
 	if (found)
 		*found = TRUE;
 
-	if (cache->ref)
-		cache->ref(data->data, cache->user_data);
+	if (data->data)
+		g_object_ref(data->data);
 
 	return data->data;
 }
@@ -207,7 +213,7 @@ gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
 /* Call the update() function on this item if it's in the cache
  * AND it's out-of-date.
  */
-void g_fscache_may_update(GFSCache *cache, char *pathname)
+void g_fscache_may_update(GFSCache *cache, const char *pathname)
 {
 	GFSCacheKey	key;
 	GFSCacheData	*data;
@@ -229,13 +235,14 @@ void g_fscache_may_update(GFSCache *cache, char *pathname)
 	{
 		cache->update(data->data, pathname, cache->user_data);
 		data->m_time = info.st_mtime;
+		data->c_time = info.st_ctime;
 		data->length = info.st_size;
 		data->mode = info.st_mode;
 	}
 }
 
 /* Call the update() function on this item iff it's in the cache. */
-void g_fscache_update(GFSCache *cache, char *pathname)
+void g_fscache_update(GFSCache *cache, const char *pathname)
 {
 	GFSCacheKey	key;
 	GFSCacheData	*data;
@@ -257,6 +264,7 @@ void g_fscache_update(GFSCache *cache, char *pathname)
 	{
 		cache->update(data->data, pathname, cache->user_data);
 		data->m_time = info.st_mtime;
+		data->c_time = info.st_ctime;
 		data->length = info.st_size;
 		data->mode = info.st_mode;
 	}
@@ -304,11 +312,10 @@ static gint cmp_stats(gconstpointer a, gconstpointer b)
 
 static void destroy_hash_entry(gpointer key, gpointer data, gpointer user_data)
 {
-	GFSCache *cache = (GFSCache *) user_data;
 	GFSCacheData *cache_data = (GFSCacheData *) data;
 	
-	if (cache->unref)
-		cache->unref(cache_data->data, cache->user_data);
+	if (cache_data->data)
+		g_object_unref(cache_data->data);
 
 	g_free(key);
 	g_free(data);
@@ -319,19 +326,17 @@ static gboolean purge_hash_entry(gpointer key, gpointer data,
 {
 	struct PurgeInfo *info = (struct PurgeInfo *) user_data;
 	GFSCacheData *cache_data = (GFSCacheData *) data;
-	GFSCache *cache = info->cache;
 
 	/* It's wasteful to remove an entry if someone else is using it */
-	if (cache->getref &&
-		cache->getref(cache_data->data, cache->user_data) > 1)
+	if (cache_data->data && cache_data->data->ref_count > 1)
 		return FALSE;
-			
+
 	if (cache_data->last_lookup <= info->now
 		&& cache_data->last_lookup >= info->now - info->age)
 		return FALSE;
 
-	if (cache->unref)
-		cache->unref(cache_data->data, cache->user_data);
+	if (cache_data->data)
+		g_object_unref(cache_data->data);
 
 	g_free(key);
 	g_free(data);
@@ -342,7 +347,7 @@ static gboolean purge_hash_entry(gpointer key, gpointer data,
 /* As for g_fscache_lookup_full, but return the GFSCacheData rather than
  * the data it contains. Doesn't increment the refcount.
  */
-static GFSCacheData *lookup_internal(GFSCache *cache, char *pathname,
+static GFSCacheData *lookup_internal(GFSCache *cache, const char *pathname,
 					FSCacheLookup lookup_type)
 {
 	struct stat 	info;
@@ -384,8 +389,8 @@ static GFSCacheData *lookup_internal(GFSCache *cache, char *pathname,
 			cache->update(data->data, pathname, cache->user_data);
 		else
 		{
-			if (cache->unref)
-				cache->unref(data->data, cache->user_data);
+			if (data->data)
+				g_object_unref(data->data);
 			data->data = NULL;
 		}
 	}
@@ -407,6 +412,7 @@ static GFSCacheData *lookup_internal(GFSCache *cache, char *pathname,
 
 init:
 	data->m_time = info.st_mtime;
+	data->c_time = info.st_ctime;
 	data->length = info.st_size;
 	data->mode = info.st_mode;
 
