@@ -1,5 +1,5 @@
 /*
- * $Id: gui_support.c,v 1.114 2004/04/27 10:34:30 tal197 Exp $
+ * $Id: gui_support.c,v 1.117 2004/07/30 17:08:39 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -51,6 +51,7 @@ gint	screen_width, screen_height;
 gint		n_monitors;
 GdkRectangle	*monitor_geom = NULL;
 gint		monitor_width, monitor_height;
+MonitorAdjacent *monitor_adjacent;
 
 static GdkAtom xa_cardinal;
 
@@ -64,26 +65,50 @@ static gint tip_timeout = 0;	/* When primed */
 static void run_error_info_dialog(GtkMessageType type, const char *message,
 				  va_list args);
 static GType simple_image_get_type(void);
+static void gui_get_monitor_adjacent(int monitor, MonitorAdjacent *adj);
 
 void gui_store_screen_geometry(GdkScreen *screen)
 {
 	gint mon;
 
+	screen_width = gdk_screen_get_width(screen);
+	screen_height = gdk_screen_get_height(screen);
+
+	if (monitor_adjacent)
+		g_free(monitor_adjacent);
+
 	monitor_width = monitor_height = G_MAXINT;
 	n_monitors = gdk_screen_get_n_monitors(screen);
 	if (monitor_geom)
 		g_free(monitor_geom);
-	monitor_geom = g_new(GdkRectangle, n_monitors);
-	for (mon = 0; mon < n_monitors; ++mon)
+	monitor_geom = g_new(GdkRectangle, n_monitors ? n_monitors : 1);
+
+	if (n_monitors)
 	{
-		gdk_screen_get_monitor_geometry(screen, mon,
-				&monitor_geom[mon]);
-		if (monitor_geom[mon].width < monitor_width)
-			monitor_width = monitor_geom[mon].width; 
-		if (monitor_geom[mon].height < monitor_height)
-			monitor_height = monitor_geom[mon].height; 
+		for (mon = 0; mon < n_monitors; ++mon)
+		{
+			gdk_screen_get_monitor_geometry(screen, mon,
+					&monitor_geom[mon]);
+			if (monitor_geom[mon].width < monitor_width)
+				monitor_width = monitor_geom[mon].width; 
+			if (monitor_geom[mon].height < monitor_height)
+				monitor_height = monitor_geom[mon].height; 
+		}
+		monitor_adjacent = g_new(MonitorAdjacent, n_monitors);
+		for (mon = 0; mon < n_monitors; ++mon)
+		{
+			gui_get_monitor_adjacent(mon, &monitor_adjacent[mon]);
+		}
 	}
-	
+	else
+	{
+		n_monitors = 1;
+		monitor_geom[0].x = monitor_geom[0].y = 0;
+		monitor_width = monitor_geom[0].width = screen_width;
+		monitor_height = monitor_geom[0].height = screen_height;
+		monitor_adjacent = g_new0(MonitorAdjacent, 1);
+	}
+
 }
 
 void gui_support_init()
@@ -95,8 +120,6 @@ void gui_support_init()
 	/* This call starts returning strange values after a while, so get
 	 * the result here during init.
 	 */
-	gdk_drawable_get_size(gdk_get_default_root_window(),
-			    &screen_width, &screen_height);
 	gui_store_screen_geometry(gdk_screen_get_default());
 
 	/* Work around the scrollbar placement bug */
@@ -320,7 +343,7 @@ void delayed_error(const char *error, ...)
 	else
 	{
 		delayed_error_data = new;
-		gtk_idle_add(error_idle_cb, &delayed_error_data);
+		g_idle_add(error_idle_cb, &delayed_error_data);
 
 		number_of_windows++;
 	}
@@ -436,17 +459,24 @@ gboolean get_pointer_xy(int *x, int *y)
 void centre_window(GdkWindow *window, int x, int y)
 {
 	int	w, h;
+	int m;
 
 	g_return_if_fail(window != NULL);
 
+	m = gdk_screen_get_monitor_at_point(gdk_screen_get_default(), x, y);
+
 	gdk_drawable_get_size(window, &w, &h);
 	
 	x -= w / 2;
 	y -= h / 2;
 
 	gdk_window_move(window,
-		CLAMP(x, DECOR_BORDER, screen_width - w - DECOR_BORDER),
-		CLAMP(y, DECOR_BORDER, screen_height - h - DECOR_BORDER));
+		CLAMP(x, DECOR_BORDER + monitor_geom[m].x,
+			monitor_geom[m].x + monitor_geom[m].width 
+			- w - DECOR_BORDER),
+		CLAMP(y, DECOR_BORDER + monitor_geom[m].y,
+			monitor_geom[m].y + monitor_geom[m].height 
+			- h - DECOR_BORDER));
 }
 
 static void run_error_info_dialog(GtkMessageType type, const char *message,
@@ -490,14 +520,14 @@ static gboolean end_wink(gpointer data)
 
 static void cancel_wink(void)
 {
-	gtk_timeout_remove(wink_timeout);
+	g_source_remove(wink_timeout);
 	end_wink(NULL);
 }
 
 static void wink_widget_died(gpointer data)
 {
 	current_wink_widget = NULL;
-	gtk_timeout_remove(wink_timeout);
+	g_source_remove(wink_timeout);
 }
 
 /* Draw a black box around this widget, briefly.
@@ -513,7 +543,7 @@ void wink_widget(GtkWidget *widget)
 	current_wink_widget = widget;
 	gtk_drag_highlight(current_wink_widget);
 	
-	wink_timeout = gtk_timeout_add(300, (GtkFunction) end_wink, NULL);
+	wink_timeout = g_timeout_add(300, (GSourceFunc) end_wink, NULL);
 
 	wink_destroy = g_signal_connect_swapped(widget, "destroy",
 				G_CALLBACK(wink_widget_died), NULL);
@@ -530,7 +560,7 @@ static gboolean idle_destroy_cb(GtkWidget *widget)
 void destroy_on_idle(GtkWidget *widget)
 {
 	gtk_widget_ref(widget);
-	gtk_idle_add((GtkFunction) idle_destroy_cb, widget);
+	g_idle_add((GSourceFunc) idle_destroy_cb, widget);
 }
 
 /* Spawn a child process (as spawn_full), and report errors.
@@ -713,10 +743,11 @@ void tooltip_show(guchar *text)
 	GtkWidget *label;
 	int	x, y, py;
 	int	w, h;
+	int m;
 
 	if (tip_timeout)
 	{
-		gtk_timeout_remove(tip_timeout);
+		g_source_remove(tip_timeout);
 		tip_timeout = 0;
 	}
 
@@ -747,16 +778,20 @@ void tooltip_show(guchar *text)
 	h = tip_widget->allocation.height;
 	gdk_window_get_pointer(NULL, &x, &py, NULL);
 
+	m = gdk_screen_get_monitor_at_point(gdk_screen_get_default(), x, py);
+	
 	x -= w / 2;
 	y = py + 12; /* I don't know the pointer height so I use a constant */
 
 	/* Now check for screen boundaries */
-	x = CLAMP(x, 0, screen_width - w);
-	y = CLAMP(y, 0, screen_height - h);
+	x = CLAMP(x, monitor_geom[m].x,
+			monitor_geom[m].x + monitor_geom[m].width - w);
+	y = CLAMP(y, monitor_geom[m].y,
+			monitor_geom[m].y + monitor_geom[m].height - h);
 
 	/* And again test if pointer is over the tooltip window */
 	if (py >= y && py <= y + h)
-		y = py - h- 2;
+		y = py - h - 2;
 	gtk_window_move(GTK_WINDOW(tip_widget), x, y);
 	gtk_widget_show(tip_widget);
 
@@ -779,11 +814,11 @@ void tooltip_prime(GtkFunction callback, GObject *object)
 	delay = now - tip_time > 2 ? 1000 : 200;
 
 	g_object_ref(object);
-	tip_timeout = gtk_timeout_add_full(delay,
-					   (GtkFunction) callback,
-					   NULL,
-					   object,
-					   g_object_unref);
+	tip_timeout = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE,
+					 delay,
+					 (GSourceFunc) callback,
+					 object,
+					 g_object_unref);
 }
 
 /* Like gtk_widget_modify_font, but copes with font_desc == NULL */
@@ -1026,25 +1061,9 @@ static void simple_image_size_request(GtkWidget      *widget,
 void render_pixbuf(GdkPixbuf *pixbuf, GdkDrawable *target, GdkGC *gc,
 		   int x, int y, int width, int height)
 {
-#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION > 1
 	gdk_draw_pixbuf(target, gc, pixbuf, 0, 0, x, y, width, height,
 		        GDK_RGB_DITHER_NORMAL, 0, 0);
 
-#else
-	static gboolean warned = FALSE;
-	if (gtk_minor_version == 0)
-		GDK_DRAWABLE_GET_CLASS(target)->_draw_pixbuf(target, gc, pixbuf,
-				   0, 0, x, y, width, height,
-				   GDK_RGB_DITHER_NORMAL, 0, 0);
-	else if (!warned)
-	{
-		delayed_error(_("Pinboard icons cannot be drawn because "
-				"ROX-Filer was compiled against GTK+-2.0 "
-				"but is running with GTK+-2.2. Please "
-				"recompile it."));
-		warned = TRUE;
-	}
-#endif
 }
 
 static gint simple_image_expose(GtkWidget *widget, GdkEventExpose *event)
@@ -1122,3 +1141,52 @@ GtkWidget *simple_image_new(GdkPixbuf *pixbuf)
 
 	return GTK_WIDGET(image);
 }
+
+/* Whether a line l1 long starting from n1 overlaps a line l2 from n2 */
+inline static gboolean gui_ranges_overlap(int n1, int l1, int n2, int l2)
+{
+	return (n1 > n2 && n1 < n2 + l2) ||
+		(n1 + l1 > n2 && n1 + l1 < n2 + l2) ||
+		(n1 <= n2 && n1 + l1 >= n2 + l2);
+}
+
+static void gui_get_monitor_adjacent(int monitor, MonitorAdjacent *adj)
+{
+	int m;
+
+	adj->left = adj->right = adj->top = adj->bottom = FALSE;
+
+	for (m = 0; m < n_monitors; ++m)
+	{
+		if (m == monitor)
+			continue;
+		if (gui_ranges_overlap(monitor_geom[m].y,
+				monitor_geom[m].height,
+				monitor_geom[monitor].y,
+				monitor_geom[monitor].height))
+		{
+			if (monitor_geom[m].x < monitor_geom[monitor].x)
+			{
+				adj->left = TRUE;
+			}
+			else if (monitor_geom[m].x > monitor_geom[monitor].x)
+			{
+				adj->right = TRUE;
+			}
+		}
+		if (gui_ranges_overlap(monitor_geom[m].x,
+				monitor_geom[m].width,
+				monitor_geom[monitor].x,
+				monitor_geom[monitor].width))
+		{
+			if (monitor_geom[m].y < monitor_geom[monitor].y)
+			{
+				adj->top = TRUE;
+			}
+			else if (monitor_geom[m].y > monitor_geom[monitor].y)
+			{
+				adj->bottom = TRUE;
+			}
+		}
+	}
+}
