@@ -22,9 +22,21 @@ typedef struct _Radios Radios;
 extern GdkFont	   	*fixed_font;
 extern gint		screen_width, screen_height;
 
+/* For Xinerama */
+extern gint		n_monitors;
+extern GdkRectangle	*monitor_geom;
+/* Smallest monitor - use for sizing windows */
+extern gint		monitor_width, monitor_height;
+typedef struct {
+	gboolean left, right, top, bottom;
+} MonitorAdjacent;
+extern MonitorAdjacent *monitor_adjacent;
+
 typedef void (*HelpFunc)(gpointer data);
 typedef const char *ParseFunc(gchar *line);
 
+void gui_store_screen_geometry(GdkScreen *screen);
+
 void gui_support_init(void);
 int get_choice(const char *title,
 	       const char *message,
@@ -41,7 +53,7 @@ gboolean get_pointer_xy(int *x, int *y);
 void centre_window(GdkWindow *window, int x, int y);
 void wink_widget(GtkWidget *widget);
 void destroy_on_idle(GtkWidget *widget);
-gboolean rox_spawn(const gchar *dir, const gchar **argv);
+gint rox_spawn(const gchar *dir, const gchar **argv);
 GtkWidget *button_new_mixed(const char *stock, const char *message);
 GtkWidget *button_new_image_text(GtkWidget *image, const char *message);
 void entry_set_error(GtkWidget *entry, gboolean error);
