@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.160 2003/01/19 19:29:04 tal197 Exp $
+ * $Id: main.c,v 1.173 2004/07/30 17:08:40 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -44,6 +44,7 @@
 #endif
 
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>		/* For rox_x_error */
 
 #include "global.h"
 
@@ -71,6 +72,7 @@
 #include "panel.h"
 #include "session.h"
 #include "minibuffer.h"
+#include "xtypes.h"
 
 int number_of_windows = 0;	/* Quit when this reaches 0 again... */
 int to_wakeup_pipe = -1;	/* Write here to get noticed */
@@ -108,6 +110,8 @@ GtkTooltips *tooltips = NULL;
 		"you must use the short versions instead.\n\n")
 #endif
 
+#define BUGS_TO "<rox-devel@lists.sourceforge.net>"
+
 #define HELP N_("Usage: ROX-Filer/AppRun [OPTION]... [FILE]...\n"	\
        "Open each directory or file listed, or the current working\n"	\
        "directory if no arguments are given.\n\n"			\
@@ -127,7 +131,7 @@ GtkTooltips *tooltips = NULL;
        "  -u, --user		show user name in each window \n"	\
        "  -v, --version		display the version information and exit\n"   \
        "  -x, --examine=FILE	FILE has changed - re-examine it\n"	\
-       "\nReport bugs to <tal197@users.sourceforge.net>.\n"		\
+       "\nReport bugs to " BUGS_TO ".\n"		\
        "Home page (including updated versions): http://rox.sourceforge.net/\n")
 
 #define SHORT_OPS "c:d:t:b:l:r:op:s:hvnux:m:D:R"
@@ -177,6 +181,7 @@ static void soap_add(xmlNodePtr body,
 static void child_died(int signum);
 static void child_died_callback(void);
 static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition);
+static void xrandr_size_change(GdkScreen *screen, gpointer user_data);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -185,6 +190,31 @@ static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition);
 /* The value that goes with an option */
 #define VALUE (*optarg == '=' ? optarg + 1 : optarg)
 
+static int rox_x_error(Display *display, XErrorEvent *error)
+{
+	  gchar buf[64];
+          
+	  XGetErrorText(display, error->error_code, buf, 63);
+
+	  g_warning ("The program '%s' received an X Window System error.\n"
+                             "This probably reflects a bug in the program.\n"
+                             "The error was '%s'.\n"
+                             "  (Details: serial %ld error_code %d request_code %d minor_code %d)\n"
+                             "  (Note to programmers: normally, X errors are reported asynchronously;\n"
+                             "   that is, you will receive the error a while after causing it.\n"
+                             "   To debug your program, run it with the --sync command line\n"
+                             "   option to change this behavior. You can then get a meaningful\n"
+                             "   backtrace from your debugger.)",
+                             g_get_prgname (),
+                             buf,
+                             error->serial, 
+                             error->error_code, 
+                             error->request_code,
+                             error->minor_code);
+          
+	  abort();
+}
+
 /* Parses the command-line to work out what the user wants to do.
  * Tries to send the request to an already-running copy of the filer.
  * If that fails, it initialises all the other modules and executes the
@@ -208,6 +238,7 @@ int main(int argc, char **argv)
 	/* Get internationalisation up and running. This requires the
 	 * choices system, to discover the user's preferred language.
 	 */
+	xtype_init();
 	choices_init();
 	options_init();
 	i18n_init();
@@ -285,6 +316,9 @@ int main(int argc, char **argv)
 		"class \"Collection\" style : gtk "
 		"\"rox-default-collection-style\"\n");
 
+	g_signal_connect(gdk_screen_get_default(), "size-changed",
+			 G_CALLBACK(xrandr_size_change), NULL);
+
 	/* Process each option in turn */
 	while (1)
 	{
@@ -373,6 +407,8 @@ int main(int argc, char **argv)
 			{
 				MIME_type *type;
 				type_init();
+				diritem_init();
+				pixmaps_init();
 				type = type_get_type(VALUE);
 				printf("%s/%s\n", type->media_type,
 						type->subtype);
@@ -444,6 +480,7 @@ int main(int argc, char **argv)
 	 */
 	if (!new_copy)
 	{
+		int fd;
 		pid_t child;
 
 		child = fork();
@@ -452,13 +489,10 @@ int main(int argc, char **argv)
 		/* Otherwise we're the child (or an error occurred - ignore
 		 * it!).
 		 */
-	}
 
-	/* Close stdin. We don't need it, and it can cause problems if
-	 * a child process wants a password, etc...
-	 */
-	{
-		int fd;
+		/* Close stdin. We don't need it, and it can cause problems if
+		 * a child process wants a password, etc...
+		 */
 		fd = open("/dev/null", O_RDONLY);
 		if (fd > 0)
 		{
@@ -498,8 +532,8 @@ int main(int argc, char **argv)
 	pipe(wakeup_pipe);
 	close_on_exec(wakeup_pipe[0], TRUE);
 	close_on_exec(wakeup_pipe[1], TRUE);
-	gtk_input_add_full(wakeup_pipe[0], GDK_INPUT_READ, wake_up_cb,
-			NULL, NULL, NULL);
+	gdk_input_add_full(wakeup_pipe[0], GDK_INPUT_READ, wake_up_cb,
+			NULL, NULL);
 	to_wakeup_pipe = wakeup_pipe[1];
 
 	/* If the pipe is full then we're going to get woken up anyway... */
@@ -531,6 +565,9 @@ int main(int argc, char **argv)
 		xmlFreeDoc(reply);
 	}
 
+	/* Try to find out why we crash with GTK 2.4 */
+	XSetErrorHandler(rox_x_error);
+
 	/* Enter the main loop, processing events until all our windows
 	 * are closed.
 	 */
@@ -595,6 +632,13 @@ static void show_features(void)
 		_("Yes")
 #else
 		_("No")
+#endif
+	       );
+	g_print("%s... %s\n", _("Binary compatibility"),
+#if defined(HAVE_APSYMBOLS_H) || defined(HAVE_APBUILD_APSYMBOLS_H)
+		_("Yes (can run with older glibc versions)")
+#else
+		_("No (apsymbols.h not found)")
 #endif
 	       );
 }
@@ -674,3 +718,11 @@ static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition)
 		dnotify_wakeup();
 #endif
 }
+
+static void xrandr_size_change(GdkScreen *screen, gpointer user_data)
+{
+	gui_store_screen_geometry(screen);
+
+	panel_update_size();
+	pinboard_update_size();
+}
