@@ -1,8 +1,8 @@
 /*
- * $Id: minibuffer.c,v 1.42 2002/01/02 15:36:04 tal197 Exp $
+ * $Id: minibuffer.c,v 1.70 2004/04/21 09:07:16 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2002, the ROX-Filer team.
+ * Copyright (C) 2003, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -23,6 +23,7 @@
 
 #include "config.h"
 
+#include <fnmatch.h>
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
@@ -35,7 +36,6 @@
 #include "global.h"
 
 #include "options.h"
-#include "collection.h"
 #include "find.h"
 #include "gui_support.h"
 #include "support.h"
@@ -46,6 +46,7 @@
 #include "action.h"
 #include "diritem.h"
 #include "type.h"
+#include "view_iface.h"
 
 static GList *shell_history = NULL;
 
@@ -55,25 +56,27 @@ static gint key_press_event(GtkWidget	*widget,
 			FilerWindow	*filer_window);
 static void changed(GtkEditable *mini, FilerWindow *filer_window);
 static gboolean find_next_match(FilerWindow *filer_window,
-				char *pattern,
+				const char *pattern,
 				int dir);
-static gboolean find_exact_match(FilerWindow *filer_window, char *pattern);
-static gboolean matches(Collection *collection, int item, char *pattern);
+static gboolean find_exact_match(FilerWindow *filer_window,
+				 const gchar *pattern);
+static gboolean matches(ViewIter *iter, const char *pattern);
 static void search_in_dir(FilerWindow *filer_window, int dir);
-static guchar *mini_contents(FilerWindow *filer_window);
+static const gchar *mini_contents(FilerWindow *filer_window);
 static void show_help(FilerWindow *filer_window);
-#ifdef GTK2
 static gboolean grab_focus(GtkWidget *minibuffer);
-#endif
+static gboolean select_if_glob(ViewIter *iter, gpointer data);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
  ****************************************************************/
 
+static Option o_filer_beep_fail, o_filer_beep_multi;
+
 void minibuffer_init(void)
 {
-	option_add_int("filer_beep_fail", 1, NULL);
-	option_add_int("filer_beep_multi", 1, NULL);
+	option_add_int(&o_filer_beep_fail, "filer_beep_fail", 1);
+	option_add_int(&o_filer_beep_multi, "filer_beep_multi", 1);
 }
 
 /* Creates the minibuffer widgets, setting the appropriate fields
@@ -95,17 +98,13 @@ void create_minibuffer(FilerWindow *filer_window)
 	mini = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(hbox), mini, TRUE, TRUE, 0);
 	gtk_widget_set_name(mini, "fixed-style");
-	gtk_signal_connect(GTK_OBJECT(mini), "key_press_event",
-			GTK_SIGNAL_FUNC(key_press_event), filer_window);
-	gtk_signal_connect(GTK_OBJECT(mini), "changed",
-			GTK_SIGNAL_FUNC(changed), filer_window);
+	g_signal_connect(mini, "key_press_event",
+			G_CALLBACK(key_press_event), filer_window);
+	g_signal_connect(mini, "changed", G_CALLBACK(changed), filer_window);
 
-#ifdef GTK2
 	/* Grabbing focus musn't select the text... */
-	gtk_signal_connect_object(GTK_OBJECT(mini), "grab-focus",
-		GTK_SIGNAL_FUNC(grab_focus),
-		GTK_OBJECT(mini));
-#endif
+	g_signal_connect_swapped(mini, "grab-focus",
+		G_CALLBACK(grab_focus), mini);
 
 	filer_window->minibuffer = mini;
 	filer_window->minibuffer_label = label;
@@ -114,50 +113,67 @@ void create_minibuffer(FilerWindow *filer_window)
 
 void minibuffer_show(FilerWindow *filer_window, MiniType mini_type)
 {
-	Collection	*collection;
 	GtkEntry	*mini;
-	int		pos = -1, i;
+	int		pos = -1;
+	ViewIter	cursor;
 	
 	g_return_if_fail(filer_window != NULL);
 	g_return_if_fail(filer_window->minibuffer != NULL);
 
 	mini = GTK_ENTRY(filer_window->minibuffer);
+	entry_set_error(filer_window->minibuffer, FALSE);
 
 	filer_window->mini_type = MINI_NONE;
 	gtk_label_set_text(GTK_LABEL(filer_window->minibuffer_label),
 			mini_type == MINI_PATH ? _("Goto:") :
 			mini_type == MINI_SHELL ? _("Shell:") :
 			mini_type == MINI_SELECT_IF ? _("Select If:") :
+			mini_type == MINI_SELECT_BY_NAME ? _("Select Named:") :
+			mini_type == MINI_FILTER ? _("Pattern:") :
 			"?");
 
-	collection = filer_window->collection;
 	switch (mini_type)
 	{
 		case MINI_PATH:
-			collection_move_cursor(collection, 0, 0);
-			filer_window->mini_cursor_base =
-					MAX(collection->cursor_item, 0);
+			view_show_cursor(filer_window->view);
+			view_get_cursor(filer_window->view, &cursor);
+			view_set_base(filer_window->view, &cursor);
+
 			gtk_entry_set_text(mini,
-					make_path(filer_window->path, "")->str);
+				make_path(filer_window->sym_path, ""));
 			if (filer_window->temp_show_hidden)
 			{
-				display_set_hidden(filer_window, FALSE);
 				filer_window->temp_show_hidden = FALSE;
+				display_update_hidden(filer_window);
 			}
 			break;
 		case MINI_SELECT_IF:
 			gtk_entry_set_text(mini, "");
 			filer_window->mini_cursor_base = -1;	/* History */
 			break;
+		case MINI_SELECT_BY_NAME:
+			gtk_entry_set_text(mini, "*.");
+			filer_window->mini_cursor_base = -1;	/* History */
+			view_select_if(filer_window->view, select_if_glob, "*.");
+			break;
+		case MINI_FILTER:
+			if(filer_window->filter!=FILER_SHOW_GLOB ||
+			   !filer_window->filter_string)
+				gtk_entry_set_text(mini, "*");
+			else
+				gtk_entry_set_text(mini,
+						  filer_window->filter_string);
+			break;
 		case MINI_SHELL:
+		{
+			DirItem *item;
+			view_get_cursor(filer_window->view, &cursor);
+			item = cursor.peek(&cursor);
 			pos = 0;
-			i = collection->cursor_item;
-			if (collection->number_selected)
+			if (view_count_selected(filer_window->view) > 0)
 				gtk_entry_set_text(mini, " \"$@\"");
-			else if (i > -1 && i < collection->number_of_items)
+			else if (item)
 			{
-				DirItem *item = (DirItem *) 
-						collection->items[i].data;
 				guchar *tmp;
 
 				tmp = g_strconcat(" ", item->leafname, NULL);
@@ -168,6 +184,7 @@ void minibuffer_show(FilerWindow *filer_window, MiniType mini_type)
 				gtk_entry_set_text(mini, "");
 			filer_window->mini_cursor_base = -1;	/* History */
 			break;
+		}
 		default:
 			g_warning("Bad minibuffer type\n");
 			return;
@@ -175,12 +192,11 @@ void minibuffer_show(FilerWindow *filer_window, MiniType mini_type)
 	
 	filer_window->mini_type = mini_type;
 
-	gtk_entry_set_position(mini, pos);
+	gtk_editable_set_position(GTK_EDITABLE(mini), pos);
 
 	gtk_widget_show_all(filer_window->minibuffer_area);
 
-	gtk_window_set_focus(GTK_WINDOW(filer_window->window),
-			filer_window->minibuffer);
+	gtk_widget_grab_focus(filer_window->minibuffer);
 }
 
 void minibuffer_hide(FilerWindow *filer_window)
@@ -188,20 +204,19 @@ void minibuffer_hide(FilerWindow *filer_window)
 	filer_window->mini_type = MINI_NONE;
 
 	gtk_widget_hide(filer_window->minibuffer_area);
-	gtk_window_set_focus(GTK_WINDOW(filer_window->window),
-			GTK_WIDGET(filer_window->collection));
+
+	gtk_widget_child_focus(filer_window->window, GTK_DIR_TAB_FORWARD);
 
 	if (filer_window->temp_show_hidden)
 	{
-		Collection *collection = filer_window->collection;
-		int i = collection->cursor_item;
-		DirItem *item = NULL;
+		DirItem *item;
+		ViewIter iter;
+
+		view_get_cursor(filer_window->view, &iter);
+		item = iter.peek(&iter);
 
-		if (i >= 0 && i < collection->number_of_items)
-			item = (DirItem *) collection->items[i].data;
-		
 		if (item == NULL || item->leafname[0] != '.')
-			display_set_hidden(filer_window, FALSE);
+		        display_update_hidden(filer_window);
 		filer_window->temp_show_hidden = FALSE;
 	}
 }
@@ -209,7 +224,7 @@ void minibuffer_hide(FilerWindow *filer_window)
 /* Insert this leafname at the cursor (replacing the selection, if any).
  * Must be in SHELL mode.
  */
-void minibuffer_add(FilerWindow *filer_window, guchar *leafname)
+void minibuffer_add(FilerWindow *filer_window, const gchar *leafname)
 {
 	guchar		*esc;
 	GtkEditable 	*edit = GTK_EDITABLE(filer_window->minibuffer);
@@ -239,30 +254,41 @@ void minibuffer_add(FilerWindow *filer_window, guchar *leafname)
 
 static void show_help(FilerWindow *filer_window)
 {
-	guchar	*message;
-
-	gtk_widget_grab_focus(filer_window->minibuffer);
-
 	switch (filer_window->mini_type)
 	{
 		case MINI_PATH:
-			message = _("Enter the name of a file and I'll display "
+			info_message(
+				_("Enter the name of a file and I'll display "
 				"it for you. Press Tab to fill in the longest "
-				"match. Escape to close the minibuffer.");
+				"match. Escape to close the minibuffer."));
 			break;
 		case MINI_SHELL:
-			message = _("Enter a shell command to execute. Click "
-				"on a file to add it to the buffer.");
+			info_message(
+				_("Enter a shell command to execute. Click "
+				"on a file to add it to the buffer."));
+			break;
+		case MINI_SELECT_BY_NAME:
+			info_message(
+				_("Enter a file name pattern to select all matching files:\n\n"
+				"? means any character\n"
+				"* means zero or more characters\n"
+				"[aA] means 'a' or 'A'\n"
+				"[a-z] means any character from a to z (lowercase)\n"
+				"*.png means any name ending in '.png'"));
 			break;
 		case MINI_SELECT_IF:
 			show_condition_help(NULL);
-			return;
+			break;
+		case MINI_FILTER:
+			info_message(
+				_("Enter a pattern to match for files to "
+				"be shown.  An empty filter turns the "
+				  "filter off."));
+			break;
 		default:
-			message = "?!?";
+			g_warning("Unknown minibuffer type!");
 			break;
 	}
-
-	delayed_error("%s", message);
 }
 
 
@@ -270,20 +296,18 @@ static void show_help(FilerWindow *filer_window)
 
 static void path_return_pressed(FilerWindow *filer_window, GdkEventKey *event)
 {
-	Collection 	*collection = filer_window->collection;
-	int		item = collection->cursor_item;
-	char		*path, *pattern;
+	const gchar	*path, *pattern;
 	int		flags = OPEN_FROM_MINI | OPEN_SAME_WINDOW;
+	ViewIter	iter;
+	DirItem		*item;
 
 	path = gtk_entry_get_text(GTK_ENTRY(filer_window->minibuffer));
-	pattern = strrchr(path, '/');
-	if (pattern)
-		pattern++;
-	else
-		pattern = path;
+	pattern = g_basename(path);
 
-	if (item == -1 || item >= collection->number_of_items ||
-			!matches(collection, item, pattern))
+	view_get_cursor(filer_window->view, &iter);
+
+	item = iter.peek(&iter);
+	if (item == NULL || !matches(&iter, pattern))
 	{
 		gdk_beep();
 		return;
@@ -292,7 +316,7 @@ static void path_return_pressed(FilerWindow *filer_window, GdkEventKey *event)
 	if ((event->state & GDK_SHIFT_MASK) != 0)
 		flags |= OPEN_SHIFT;
 
-	filer_openitem(filer_window, item, flags);
+	filer_openitem(filer_window, &iter, flags);
 }
 
 /* Use the cursor item to fill in the minibuffer.
@@ -302,23 +326,22 @@ static void path_return_pressed(FilerWindow *filer_window, GdkEventKey *event)
 static void complete(FilerWindow *filer_window)
 {
 	GtkEntry	*entry;
-	Collection 	*collection = filer_window->collection;
-	int		cursor = collection->cursor_item;
-	DirItem 	*item;
+	DirItem 	*item, *other;
 	int		shortest_stem = -1;
 	int		current_stem;
-	int		other;
-	guchar		*text, *leaf;
+	const gchar	*text, *leaf;
+	ViewIter	cursor, iter;
 
-	if (cursor < 0 || cursor >= collection->number_of_items)
+	view_get_cursor(filer_window->view, &cursor);
+	item = cursor.peek(&cursor);
+
+	if (!item)
 	{
 		gdk_beep();
 		return;
 	}
 
 	entry = GTK_ENTRY(filer_window->minibuffer);
-	
-	item = (DirItem *) collection->items[cursor].data;
 
 	text = gtk_entry_get_text(entry);
 	leaf = strrchr(text, '/');
@@ -329,7 +352,7 @@ static void complete(FilerWindow *filer_window)
 	}
 
 	leaf++;
-	if (!matches(collection, cursor, leaf))
+	if (!matches(&cursor, leaf))
 	{
 		gdk_beep();
 		return;
@@ -340,17 +363,17 @@ static void complete(FilerWindow *filer_window)
 	/* Find the longest other match of this name. It it's longer than
 	 * the currently entered text then complete only up to that length.
 	 */
-	for (other = 0; other < collection->number_of_items; other++)
+	view_get_iter(filer_window->view, &iter, 0);
+	while ((other = iter.next(&iter)))
 	{
-		DirItem *other_item = (DirItem *) collection->items[other].data;
 		int	stem = 0;
 
-		if (other == cursor)
+		if (iter.i == cursor.i)	/* XXX */
 			continue;
 
-		while (other_item->leafname[stem] && item->leafname[stem])
+		while (other->leafname[stem] && item->leafname[stem])
 		{
-			if (other_item->leafname[stem] != item->leafname[stem])
+			if (other->leafname[stem] != item->leafname[stem])
 				break;
 			stem++;
 		}
@@ -364,112 +387,114 @@ static void complete(FilerWindow *filer_window)
 	if (current_stem == shortest_stem)
 	{
 		/* We didn't add anything... */
-		if (option_get_int("filer_beep_fail"))
+		if (o_filer_beep_fail.int_value)
 			gdk_beep();
 	}
 	else if (current_stem < shortest_stem)
 	{
 		guchar	*extra;
+		gint tmp_pos;
 
 		extra = g_strndup(item->leafname + current_stem,
 				shortest_stem - current_stem);
-		gtk_entry_append_text(entry, extra);
-		g_free(extra);
-#ifdef GTK2
-		gtk_entry_set_position(entry, -1);
-#endif
 
-		if (option_get_int("filer_beep_multi"))
+		tmp_pos = entry->text_length;
+		gtk_editable_insert_text(GTK_EDITABLE(entry), extra, -1,
+					 &tmp_pos);
+		
+		g_free(extra);
+		gtk_editable_set_position(GTK_EDITABLE(entry), -1);
+
+		if (o_filer_beep_multi.int_value)
 			gdk_beep();
 	}
 	else
 	{
-		GString	*new;
+		const guchar *new;
 
-		new = make_path(filer_window->path, item->leafname);
+		new = make_path(filer_window->sym_path, item->leafname);
 
 		if (item->base_type == TYPE_DIRECTORY &&
 				(item->flags & ITEM_FLAG_APPDIR) == 0)
-			g_string_append_c(new, '/');
+			new = make_path(new, "");
 
-		gtk_entry_set_text(entry, new->str);
-#ifdef GTK2
-		gtk_entry_set_position(entry, -1);
-#endif
+		gtk_entry_set_text(entry, new);
+		gtk_editable_set_position(GTK_EDITABLE(entry), -1);
 	}
 }
 
-/* This is an idle function because Gtk+ 2.0 changes text in a entry
- * with two signals; one to blank it and one to put the new text in.
- */
-static gboolean path_changed(gpointer data)
+static void path_changed(FilerWindow *filer_window)
 {
-	FilerWindow *filer_window = (FilerWindow *) data;
 	GtkWidget *mini = filer_window->minibuffer;
-	char	*new, *leaf;
-	char	*path, *real;
+	const char	*new, *leaf;
+	char		*path;
+	gboolean	error = FALSE;
 
 	new = gtk_entry_get_text(GTK_ENTRY(mini));
 
+	if (!*new)
+	{
+		/* Entry may be blank because we're in the middle of changing
+		 * to something else...
+		 */
+		entry_set_error(mini, FALSE);
+		return;
+	}
+
 	leaf = g_basename(new);
 	if (leaf == new)
 		path = g_strdup("/");
 	else
-		path = g_dirname(new);
-	real = pathdup(path);
-	g_free(path);
+		path = g_path_get_dirname(new);
 
-	if (strcmp(real, filer_window->path) != 0)
+	if (strcmp(path, filer_window->sym_path) != 0)
 	{
 		/* The new path is in a different directory */
 		struct stat info;
 
-		if (mc_stat(real, &info) == 0 && S_ISDIR(info.st_mode))
-			filer_change_to(filer_window, real, leaf);
+		if (stat_with_timeout(path, &info) == 0 &&
+				S_ISDIR(info.st_mode))
+		{
+			filer_change_to(filer_window, path, leaf);
+		}
 		else
-			gdk_beep();
+			error = TRUE;
 	}
 	else
 	{
 		if (*leaf == '.')
 		{
-			if (!filer_window->show_hidden)
-			{
-				filer_window->temp_show_hidden = TRUE;
-				display_set_hidden(filer_window, TRUE);
-			}
-		}
-		else if (filer_window->temp_show_hidden)
-		{
-			display_set_hidden(filer_window, FALSE);
-			filer_window->temp_show_hidden = FALSE;
+			filer_window->temp_show_hidden = TRUE;
+			display_update_hidden(filer_window);
 		}
 		
 		if (find_exact_match(filer_window, leaf) == FALSE &&
 		    find_next_match(filer_window, leaf, 0) == FALSE)
-			gdk_beep();
+			error = TRUE;
 	}
 		
-	g_free(real);
+	g_free(path);
 
-	return FALSE;
+	entry_set_error(mini, error);
 }
 
 /* Look for an exact match, and move the cursor to it if found.
  * TRUE on success.
  */
-static gboolean find_exact_match(FilerWindow *filer_window, char *pattern)
+static gboolean find_exact_match(FilerWindow *filer_window,
+				 const gchar *pattern)
 {
-	Collection 	*collection = filer_window->collection;
-	int		i;
+	DirItem		*item;
+	ViewIter	iter;
+	ViewIface	*view = filer_window->view;
 
-	for (i = 0; i < collection->number_of_items; i++)
+	view_get_iter(view, &iter, 0);
+
+	while ((item = iter.next(&iter)))
 	{
-		DirItem *item = (DirItem *) collection->items[i].data;
-
 		if (strcmp(item->leafname, pattern) == 0)
 		{
-			collection_set_cursor_item(collection, i);
+			view_cursor_to_iter(view, &iter);
 			return TRUE;
 		}
 	}
@@ -477,56 +502,55 @@ static gboolean find_exact_match(FilerWindow *filer_window, char *pattern)
 	return FALSE;
 }
 
-/* Find the next item in the collection that matches 'pattern'. Start from
- * mini_cursor_base and loop at either end. dir is 1 for a forward search,
+/* Find the next item in the view that matches 'pattern'. Start from
+ * cursor_base and loop at either end. dir is 1 for a forward search,
  * -1 for backwards. 0 means forwards, but may stay the same.
  *
- * Does not automatically update mini_cursor_base.
+ * Does not automatically update cursor_base.
  *
  * Returns TRUE if a match is found.
  */
 static gboolean find_next_match(FilerWindow *filer_window,
-				char *pattern,
+				const char *pattern,
 				int dir)
 {
-	Collection *collection = filer_window->collection;
-	int 	   base = filer_window->mini_cursor_base;
-	int 	   item = base;
-	gboolean   retval = TRUE;
+	ViewIface  *view = filer_window->view;
+	ViewIter   iter;
 
-	if (collection->number_of_items < 1)
+	if (view_count_items(view) < 1)
 		return FALSE;
 
-	if (base < 0 || base>= collection->number_of_items)
-		filer_window->mini_cursor_base = base = 0;
+	view_get_iter(view, &iter,
+		VIEW_ITER_FROM_BASE |
+		(dir >= 0 ? 0 : VIEW_ITER_BACKWARDS));
 
-	do
+	if (dir != 0)
+		iter.next(&iter);	/* Don't look at the base itself */
+
+	while (iter.next(&iter))
 	{
-		/* Step to the next item */
-		item += dir;
-
-		if (item >= collection->number_of_items)
-			item = 0;
-		else if (item < 0)
-			item = collection->number_of_items - 1;
-
-		if (dir == 0)
-			dir = 1;
-		else if (item == base)
+		if (matches(&iter, pattern))
 		{
-			retval = FALSE;
-			break;		/* No (other) matches at all */
+			view_cursor_to_iter(view, &iter);
+			return TRUE;
 		}
-	} while (!matches(collection, item, pattern));
+	}
 
-	collection_set_cursor_item(collection, item);
+	/* No matches (except possibly base itself) */
+	view_get_iter(view, &iter,
+		VIEW_ITER_FROM_BASE | VIEW_ITER_ONE_ONLY |
+		(dir >= 0 ? 0 : VIEW_ITER_BACKWARDS));
 
-	return retval;
+	view_cursor_to_iter(view, &iter);
+
+	return FALSE;
 }
 
-static gboolean matches(Collection *collection, int item_number, char *pattern)
+static gboolean matches(ViewIter *iter, const char *pattern)
 {
-	DirItem *item = (DirItem *) collection->items[item_number].data;
+	DirItem *item;
+	
+	item = iter->peek(iter);
 	
 	return strncmp(item->leafname, pattern, strlen(pattern)) == 0;
 }
@@ -534,23 +558,22 @@ static gboolean matches(Collection *collection, int item_number, char *pattern)
 /* Find next match and set base for future matches. */
 static void search_in_dir(FilerWindow *filer_window, int dir)
 {
-	char	*path, *pattern;
+	const char *path, *pattern;
+	ViewIter iter;
 
 	path = gtk_entry_get_text(GTK_ENTRY(filer_window->minibuffer));
-	pattern = strrchr(path, '/');
-	if (pattern)
-		pattern++;
-	else
-		pattern = path;
+	pattern = g_basename(path);
 	
-	filer_window->mini_cursor_base = filer_window->collection->cursor_item;
+	view_get_cursor(filer_window->view, &iter);
+	view_set_base(filer_window->view, &iter);
 	find_next_match(filer_window, pattern, dir);
-	filer_window->mini_cursor_base = filer_window->collection->cursor_item;
+	view_get_cursor(filer_window->view, &iter);
+	view_set_base(filer_window->view, &iter);
 }
 
 /*			SHELL COMMANDS			*/
 
-static void add_to_history(guchar *line)
+static void add_to_history(const gchar *line)
 {
 	guchar 	*last;
 	
@@ -591,8 +614,8 @@ static guchar *best_match(FilerWindow *filer_window, glob_t *matches)
 				longest = j;
 	}
 
-	path_len = strlen(filer_window->path);
-	if (strncmp(filer_window->path, first, path_len) == 0 &&
+	path_len = strlen(filer_window->sym_path);
+	if (strncmp(filer_window->sym_path, first, path_len) == 0 &&
 			first[path_len] == '/' && first[path_len + 1])
 	{
 		path = g_strndup(first + path_len + 1, longest - path_len - 1);
@@ -615,9 +638,8 @@ static guchar *best_match(FilerWindow *filer_window, glob_t *matches)
 
 static void shell_tab(FilerWindow *filer_window)
 {
+	const gchar	*entry;
 	int	i;
-	guchar	*entry;
-	guchar	quote;
 	int	pos;
 	GString	*leaf;
 	glob_t	matches;
@@ -627,7 +649,6 @@ static void shell_tab(FilerWindow *filer_window)
 	pos = gtk_editable_get_position(GTK_EDITABLE(filer_window->minibuffer));
 	leaf = g_string_new(NULL);
 
-	quote = '\0';
 	for (i = 0; i < pos; i++)
 	{
 		guchar	c = entry[i];
@@ -644,17 +665,15 @@ static void shell_tab(FilerWindow *filer_window)
 			c = entry[++i];
 		else if (c == '"' || c == '\'')
 		{
-			guchar	cc;
-
 			for (++i; i < pos; i++)
 			{
-				cc = entry[i];
+				guchar cc = entry[i];
 
 				if (cc == '\\' && i + 1 < pos)
 					cc = entry[++i];
-				else if (entry[i] == c)
+				else if (cc == c)
 					break;
-				g_string_append_c(leaf, entry[i]);
+				g_string_append_c(leaf, cc);
 			}
 			continue;
 		}
@@ -668,7 +687,7 @@ static void shell_tab(FilerWindow *filer_window)
 	if (leaf->str[0] != '/' && leaf->str[0] != '~')
 	{
 		g_string_prepend_c(leaf, '/');
-		g_string_prepend(leaf, filer_window->path);
+		g_string_prepend(leaf, filer_window->sym_path);
 	}
 
 	g_string_append_c(leaf, '*');
@@ -703,14 +722,22 @@ static void shell_tab(FilerWindow *filer_window)
 	g_string_free(leaf, TRUE);
 }
 
+static void run_child(gpointer unused)
+{
+	/* Ensure output is noticed - send stdout to stderr */
+	dup2(STDERR_FILENO, STDOUT_FILENO);
+	close_on_exec(STDOUT_FILENO, FALSE);
+}
+
 /* Either execute the command or make it the default run action */
 static void shell_return_pressed(FilerWindow *filer_window)
 {
 	GPtrArray	*argv;
-	int		i;
-	guchar		*entry;
-	Collection	*collection = filer_window->collection;
-	int		child;
+	const gchar	*entry;
+	GError		*error = NULL;
+	pid_t		child;
+	DirItem		*item;
+	ViewIter	iter;
 
 	entry = mini_contents(filer_window);
 
@@ -722,44 +749,32 @@ static void shell_return_pressed(FilerWindow *filer_window)
 	argv = g_ptr_array_new();
 	g_ptr_array_add(argv, "sh");
 	g_ptr_array_add(argv, "-c");
-	g_ptr_array_add(argv, entry);
+	g_ptr_array_add(argv, (gchar *) entry);
 	g_ptr_array_add(argv, "sh");
 
-	for (i = 0; i < collection->number_of_items; i++)
+	view_get_iter(filer_window->view, &iter, 0);
+	while ((item = iter.next(&iter)))
 	{
-		DirItem *item = (DirItem *) collection->items[i].data;
-		if (collection->items[i].selected)
+		if (view_get_selected(filer_window->view, &iter))
 			g_ptr_array_add(argv, item->leafname);
 	}
 	
 	g_ptr_array_add(argv, NULL);
 
-	child = fork();
-
-	switch (child)
+	if (!g_spawn_async_with_pipes(filer_window->sym_path,
+			(gchar **) argv->pdata, NULL,
+			G_SPAWN_DO_NOT_REAP_CHILD |
+			G_SPAWN_SEARCH_PATH,
+			run_child, NULL,	/* Child setup fn */
+			&child,			/* Child PID */
+			NULL, NULL, NULL,	/* Standard pipes */
+			&error))
 	{
-		case -1:
-			delayed_error(_("Failed to create child process"));
-			break;
-		case 0:	/* Child */
-			/* Ensure output is noticed - send stdout to stderr */
-			dup2(STDERR_FILENO, STDOUT_FILENO);
-			close_on_exec(STDOUT_FILENO, FALSE);
-			if (chdir(filer_window->path))
-				g_printerr("chdir(%s) failed: %s\n",
-						filer_window->path,
-						g_strerror(errno));
-			execvp((char *) argv->pdata[0],
-				(char **) argv->pdata);
-			g_printerr("execvp(%s, ...) failed: %s\n",
-					(char *) argv->pdata[0],
-					g_strerror(errno));
-			_exit(0);
-		default:
-			on_child_death(child,
-					(CallbackFn) shell_done, filer_window);
-			break;
+		delayed_error("%s", error ? error->message : "(null)");
+		g_error_free(error);
 	}
+	else
+		on_child_death(child, (CallbackFn) shell_done, filer_window);
 
 	g_ptr_array_free(argv, TRUE);
 
@@ -792,13 +807,32 @@ static void shell_recall(FilerWindow *filer_window, int dir)
 
 /*			SELECT IF			*/
 
+typedef struct {
+	FindInfo info;
+	FilerWindow *filer_window;
+	FindCondition *cond;
+} SelectData;
+
+static gboolean select_if_test(ViewIter *iter, gpointer user_data)
+{
+	DirItem *item;
+	SelectData *data = user_data;
+
+	item = iter->peek(iter);
+	g_return_val_if_fail(item != NULL, FALSE);
+
+	data->info.leaf = item->leafname;
+	data->info.fullpath = make_path(data->filer_window->sym_path,
+					data->info.leaf);
+
+	return mc_lstat(data->info.fullpath, &data->info.stats) == 0 &&
+			find_test_condition(data->cond, &data->info);
+}
+
 static void select_return_pressed(FilerWindow *filer_window, guint etime)
 {
-	FindCondition	*cond;
-	int		i, n;
-	guchar		*entry;
-	Collection	*collection = filer_window->collection;
-	FindInfo	info;
+	const gchar	*entry;
+	SelectData	data;
 
 	entry = mini_contents(filer_window);
 
@@ -807,48 +841,39 @@ static void select_return_pressed(FilerWindow *filer_window, guint etime)
 
 	add_to_history(entry);
 
-	cond = find_compile(entry);
-	if (!cond)
+	data.cond = find_compile(entry);
+	if (!data.cond)
 	{
 		delayed_error(_("Invalid Find condition"));
 		return;
 	}
 
-	info.now = time(NULL);
-	info.prune = FALSE;	/* (don't care) */
-	n = collection->number_of_items;
+	data.info.now = time(NULL);
+	data.info.prune = FALSE;	/* (don't care) */
+	data.filer_window = filer_window;
 
-	collection->block_selection_changed++;
+	view_select_if(filer_window->view, select_if_test, &data);
 
-	/* If an item at the start is selected then we could lose the
-	 * primary selection after checking that item and then need to
-	 * gain it again at the end. Therefore, if anything is selected
-	 * then select the last item until the end of the search.
-	 */
-	if (collection->number_selected)
-		collection_select_item(collection, n - 1);
-	
-	for (i = 0; i < n; i++)
-	{
-		DirItem *item = (DirItem *) collection->items[i].data;
-
-		info.leaf = item->leafname;
-		info.fullpath = make_path(filer_window->path, info.leaf)->str;
-
-		if (lstat(info.fullpath, &info.stats) == 0 &&
-				find_test_condition(cond, &info))
-			collection_select_item(collection, i);
-		else
-			collection_unselect_item(collection, i);
-	}
-
-	find_condition_free(cond);
-
-	collection_unblock_selection_changed(collection, etime, TRUE);
+	find_condition_free(data.cond);
 out:
 	minibuffer_hide(filer_window);
 }
 
+static void filter_return_pressed(FilerWindow *filer_window, guint etime)
+{
+	const gchar	*entry;
+
+	entry = mini_contents(filer_window);
+
+	if (entry && *entry && strcmp(entry, "*")!=0) {
+		display_set_filter(filer_window, FILER_SHOW_GLOB,
+				   entry);
+	} else {
+		display_set_filter(filer_window, FILER_SHOW_ALL, NULL);
+	}
+	minibuffer_hide(filer_window);
+}
+
 
 /*			EVENT HANDLERS			*/
 
@@ -860,7 +885,7 @@ static gint key_press_event(GtkWidget	*widget,
 	{
 		if (filer_window->mini_type == MINI_SHELL)
 		{
-			guchar	*line;
+			const gchar *line;
 			
 			line = mini_contents(filer_window);
 			if (line)
@@ -935,28 +960,74 @@ static gint key_press_event(GtkWidget	*widget,
 					return FALSE;
 			}
 			break;
+		case MINI_SELECT_BY_NAME:
+			switch (event->keyval)
+			{
+				case GDK_Up:
+					filer_next_selected(filer_window, -1);
+					break;
+				case GDK_Down:
+					filer_next_selected(filer_window, 1);
+					break;
+				case GDK_Tab:
+					break;
+				case GDK_Return:
+				case GDK_KP_Enter:
+					minibuffer_hide(filer_window);
+					break;
+				default:
+					return FALSE;
+			}
+			break;
+
+	        case MINI_FILTER:
+			switch (event->keyval)
+			{
+				case GDK_Return:
+				case GDK_KP_Enter:
+					filter_return_pressed(filer_window,
+								event->time);
+					break;
+				default:
+					return FALSE;
+			}
+			break;
 		default:
 			break;
 	}
 
-#ifndef GTK2
-	gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
-#endif
 	return TRUE;
 }
 
+static gboolean select_if_glob(ViewIter *iter, gpointer data)
+{
+	DirItem *item;
+	const char *pattern = (char *) data;
+
+	item = iter->peek(iter);
+	g_return_val_if_fail(item != NULL, FALSE);
+
+	return fnmatch(pattern, item->leafname, 0) == 0;
+}
+
 static void changed(GtkEditable *mini, FilerWindow *filer_window)
 {
 	switch (filer_window->mini_type)
 	{
 		case MINI_PATH:
-			gtk_idle_add(path_changed, filer_window);
+			path_changed(filer_window);
 			return;
 		case MINI_SELECT_IF:
 			set_find_string_colour(GTK_WIDGET(mini), 
 				gtk_entry_get_text(
 					GTK_ENTRY(filer_window->minibuffer)));
 			return;
+		case MINI_SELECT_BY_NAME:
+			view_select_if(filer_window->view,
+					select_if_glob,
+					(gpointer) gtk_entry_get_text(
+					      GTK_ENTRY(filer_window->minibuffer)));
+			return;
 		default:
 			break;
 	}
@@ -965,20 +1036,22 @@ static void changed(GtkEditable *mini, FilerWindow *filer_window)
 /* Returns a string (which must NOT be freed), or NULL if the buffer
  * is blank (whitespace only).
  */
-static guchar *mini_contents(FilerWindow *filer_window)
+static const gchar *mini_contents(FilerWindow *filer_window)
 {
-	guchar	*entry, *c;
+	const gchar *entry, *c;
 
 	entry = gtk_entry_get_text(GTK_ENTRY(filer_window->minibuffer));
 
 	for (c = entry; *c; c++)
-		if (!isspace(*c))
+		if (!g_ascii_isspace(*c))
 			return entry;
 
 	return NULL;
 }
 
-#ifdef GTK2
+/* This is a really ugly hack to get around Gtk+-2.0's broken auto-select
+ * behaviour.
+ */
 static gboolean grab_focus(GtkWidget *minibuffer)
 {
 	GtkWidgetClass *class;
@@ -987,8 +1060,9 @@ static gboolean grab_focus(GtkWidget *minibuffer)
 
 	class->grab_focus(minibuffer);
 
-	gtk_signal_emit_stop_by_name(GTK_OBJECT(minibuffer), "grab_focus");
+	g_signal_stop_emission(minibuffer,
+		g_signal_lookup("grab_focus", G_OBJECT_TYPE(minibuffer)), 0);
+
 
 	return 1;
 }
-#endif
