@@ -1,5 +1,5 @@
 /*
- * $Id: pixmaps.c,v 1.114 2004/03/24 20:13:16 kerofin Exp $
+ * $Id: pixmaps.c,v 1.116 2004/07/30 17:08:41 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -134,7 +134,7 @@ void pixmaps_init(void)
 
 	pixmap_cache = g_fscache_new((GFSLoadFunc) image_from_file, NULL, NULL);
 
-	gtk_timeout_add(10000, purge, NULL);
+	g_timeout_add(10000, purge, NULL);
 
 	factory = gtk_icon_factory_new();
 	for (i = 0; i < G_N_ELEMENTS(stocks); i++)
@@ -430,7 +430,9 @@ static void save_thumbnail(const char *pathname, GdkPixbuf *full)
 	smtime = g_strdup_printf("%ld", (long) info.st_mtime);
 
 	path = pathdup(pathname);
-	uri = g_strconcat("file://", path, NULL);
+	uri = g_filename_to_uri(path, NULL, NULL);
+	if(!uri)
+	        uri=g_strconcat("file://", path, NULL);
 	md5 = md5_hash(uri);
 	g_free(path);
 		
@@ -484,7 +486,9 @@ static gchar *thumbnail_path(const char *path)
 	GString *to;
 	gchar *ans;
 	
-	uri = g_strconcat("file://", path, NULL);
+	uri = g_filename_to_uri(path, NULL, NULL);
+	if(!uri)
+	       uri = g_strconcat("file://", path, NULL);
 	md5 = md5_hash(uri);
 		
 	to = g_string_new(home_dir);
@@ -581,7 +585,9 @@ static GdkPixbuf *get_thumbnail_for(const char *pathname)
 	struct stat info;
 
 	path = pathdup(pathname);
-	uri = g_strconcat("file://", path, NULL);
+	uri = g_filename_to_uri(path, NULL, NULL);
+	if(!uri)
+	        uri = g_strconcat("file://", path, NULL);
 	md5 = md5_hash(uri);
 	g_free(uri);
 	
