@@ -1,6 +1,4 @@
 /*
- * $Id: pixmaps.h,v 1.35 2001/12/22 19:32:12 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
  */
@@ -10,12 +8,6 @@
 
 #include <gtk/gtk.h>
 
-#ifndef GTK2
-# ifndef PIXMAPS_C
-  typedef struct _GdkPixbuf GdkPixbuf;
-# endif
-#endif
-
 extern GFSCache *pixmap_cache;
 
 extern MaskedPixmap *im_error;
@@ -24,23 +16,27 @@ extern MaskedPixmap *im_symlink;
 
 extern MaskedPixmap *im_unmounted;
 extern MaskedPixmap *im_mounted;
-extern MaskedPixmap *im_multiple;
 extern MaskedPixmap *im_exec_file;
 extern MaskedPixmap *im_appdir;
 
-extern MaskedPixmap *im_help;
 extern MaskedPixmap *im_dirs;
 
-/* Note: don't change these two - thumbnails saving assumes 96x96 */
+/* If making the huge size larger, be sure to update SMALL_IMAGE_THRESHOLD! */
 #define HUGE_WIDTH 96
 #define HUGE_HEIGHT 96
 
-#define ICON_HEIGHT 48
+#define ICON_HEIGHT 52
 #define ICON_WIDTH 48
 
 #define SMALL_HEIGHT 18
 #define SMALL_WIDTH 22
 
+typedef struct _MaskedPixmapClass MaskedPixmapClass;
+
+struct _MaskedPixmapClass {
+	GObjectClass parent;
+};
+
 /* When a MaskedPixmap is created we load the image from disk and
  * scale the pixbuf down to the 'huge' size (if it's bigger).
  * The 'normal' pixmap and mask are created automatically - you have
@@ -51,31 +47,31 @@ extern MaskedPixmap *im_dirs;
  */
 struct _MaskedPixmap
 {
-	int		ref;
-	GdkPixbuf	*huge_pixbuf;	/* 'Huge' source image */
+	GObject		object;
+
+	GdkPixbuf	*src_pixbuf;	/* Limited to 'huge' size */
+
+	/* If huge_pixbuf is NULL then call pixmap_make_huge() */
+	GdkPixbuf	*huge_pixbuf;
+	GdkPixbuf	*huge_pixbuf_lit;
 	int		huge_width, huge_height;
 
-	/* If huge_pixmap is NULL then call pixmap_make_huge() */
-	GdkPixmap	*huge_pixmap;	/* Huge image */
-	GdkBitmap	*huge_mask;
-
-	GdkPixmap	*pixmap;	/* Normal size image (always valid) */
-	GdkBitmap	*mask;
+	GdkPixbuf	*pixbuf;	/* Normal size image, always valid */
+	GdkPixbuf	*pixbuf_lit;
 	int		width, height;
 
-	/* If sm_pixmap is NULL then call pixmap_make_small() */
-	GdkPixmap	*sm_pixmap;	/* Small image */
-	GdkBitmap	*sm_mask;
+	/* If sm_pixbuf is NULL then call pixmap_make_small() */
+	GdkPixbuf	*sm_pixbuf;
+	GdkPixbuf	*sm_pixbuf_lit;
 	int		sm_width, sm_height;
 };
 
 void pixmaps_init(void);
-void pixmap_ref(MaskedPixmap *mp);
-void pixmap_unref(MaskedPixmap *mp);
 void pixmap_make_huge(MaskedPixmap *mp);
 void pixmap_make_small(MaskedPixmap *mp);
-MaskedPixmap *load_pixmap(char *name);
-MaskedPixmap *image_from_pixbuf(GdkPixbuf *full_size);
-void pixmap_background_thumb(gchar *path, GFunc callback, gpointer data);
+MaskedPixmap *load_pixmap(const char *name);
+void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data);
+MaskedPixmap *masked_pixmap_new(GdkPixbuf *full_size);
+GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h);
 
 #endif /* _PIXMAP_H */
