@@ -1,5 +1,5 @@
 /*
- * $Id: remote.c,v 1.54 2004/03/02 17:50:46 kerofin Exp $
+ * $Id: remote.c,v 1.55 2004/07/30 17:08:41 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -1029,7 +1029,7 @@ static void soap_send(GtkWidget *from, GdkAtom prop, GdkWindow *dest)
 	gdk_event_send_client_message((GdkEvent *) &event,
 				      GDK_WINDOW_XWINDOW(dest));
 
-	gtk_timeout_add(10000, too_slow, NULL);
+	g_timeout_add(10000, too_slow, NULL);
 			
 	gtk_main();
 }
