@@ -1,5 +1,5 @@
 /*
- * $Id: run.c,v 1.65 2004/01/21 20:03:16 tal197 Exp $
+ * $Id: run.c,v 1.66 2004/07/30 17:08:41 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -183,8 +183,8 @@ void run_with_data(const char *path, gpointer data, gulong length)
 			memcpy(pd->data, data, length);
 			pd->length = length;
 			pd->sent = 0;
-			pd->tag = gtk_input_add_full(fds[1], GDK_INPUT_WRITE,
-						write_data, NULL, pd, NULL);
+			pd->tag = gdk_input_add_full(fds[1], GDK_INPUT_WRITE,
+						write_data, pd, NULL);
 			break;
 	}
 
