@@ -1,5 +1,5 @@
 /*
- * $Id: support.c,v 1.111 2004/04/27 10:34:30 tal197 Exp $
+ * $Id: support.c,v 1.112 2004/08/01 09:45:53 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -827,7 +827,7 @@ struct _MD5Context {
 };
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-void byteSwap(guint32 *buf, unsigned words)
+static void byteSwap(guint32 *buf, unsigned words)
 {
 	md5byte *p = (md5byte *)buf;
 
