@@ -1,5 +1,5 @@
 /*
- * $Id: toolbar.c,v 1.71 2003/04/15 20:09:26 tal197 Exp $
+ * $Id: toolbar.c,v 1.76 2004/05/14 10:43:59 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -42,6 +42,7 @@
 #include "diritem.h"
 #include "view_iface.h"
 #include "bookmarks.h"
+#include "gui_support.h"
 
 typedef struct _Tool Tool;
 
@@ -60,6 +61,7 @@ struct _Tool {
 Option o_toolbar, o_toolbar_info, o_toolbar_disable;
 
 static GtkTooltips *tooltips = NULL;
+static FilerWindow *filer_window_being_counted;
 
 /* TRUE if the button presses (or released) should open a new window,
  * rather than reusing the existing one.
@@ -84,6 +86,10 @@ static void toolbar_details_clicked(GtkWidget *widget,
 				    FilerWindow *filer_window);
 static void toolbar_hidden_clicked(GtkWidget *widget,
 				   FilerWindow *filer_window);
+static void toolbar_select_clicked(GtkWidget *widget,
+				   FilerWindow *filer_window);
+static void toolbar_sort_clicked(GtkWidget *widget,
+				   FilerWindow *filer_window);
 static GtkWidget *add_button(GtkWidget *bar, Tool *tool,
 				FilerWindow *filer_window);
 static GtkWidget *create_toolbar(FilerWindow *filer_window);
@@ -138,10 +144,18 @@ static Tool all_tools[] = {
 	 toolbar_details_clicked, DROP_NONE, TRUE,
 	 FALSE},
 	
+	{N_("Sort"), GTK_STOCK_SORT_ASCENDING, N_("Change sort criteria"),
+	 toolbar_sort_clicked, DROP_NONE, FALSE,
+	 FALSE},
+	
 	{N_("Hidden"), ROX_STOCK_SHOW_HIDDEN, N_("Show/hide hidden files"),
 	 toolbar_hidden_clicked, DROP_NONE, TRUE,
 	 FALSE},
 	
+	{N_("Select"), ROX_STOCK_SELECT, N_("Select all/invert selection"),
+	 toolbar_select_clicked, DROP_NONE, FALSE,
+	 FALSE},
+ 	
 	{N_("Help"), GTK_STOCK_HELP, N_("Show ROX-Filer help"),
 	 toolbar_help_clicked, DROP_NONE, TRUE,
 	 FALSE},
@@ -193,11 +207,14 @@ void toolbar_update_info(FilerWindow *filer_window)
 			return;
 		}
 
-		if (!filer_window->show_hidden)
+		if (!(filer_window->show_hidden ||
+		      filer_window->temp_show_hidden) ||
+		    filer_window->filter!=FILER_SHOW_ALL)
 		{
 			GHashTable *hash = filer_window->directory->known_items;
 			int	   tally = 0;
 
+			filer_window_being_counted=filer_window;
 			g_hash_table_foreach(hash, tally_items, &tally);
 
 			if (tally)
@@ -292,7 +309,7 @@ static void toolbar_refresh_clicked(GtkWidget *widget,
 		filer_opendir(filer_window->sym_path, filer_window, NULL);
 	}
 	else
-		full_refresh();
+		filer_refresh(filer_window);
 }
 
 static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -380,6 +397,60 @@ static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	display_change_size(filer_window, bev->button == 1);
 }
 
+static void toolbar_sort_clicked(GtkWidget *widget,
+				    FilerWindow *filer_window)
+{
+	GdkEventButton	*bev;
+	int i, current, next, next_wrapped;
+	gboolean adjust;
+	GtkSortType dir;
+	gchar *tip;
+
+	static const SortType sorts[]={
+		SORT_NAME, SORT_TYPE, SORT_DATE, SORT_SIZE,
+		SORT_OWNER, SORT_GROUP, 
+	};
+	static const char *sort_names[] = {
+		N_("Sort by name"), N_("Sort by type"), N_("Sort by date"), 
+		N_("Sort by size"), N_("Sort by owner"), N_("Sort by group"), 
+	};
+
+	bev = (GdkEventButton *) gtk_get_current_event();
+	adjust = (bev->button != 1) && bev->type == GDK_BUTTON_RELEASE;
+
+	current = -1;
+	dir = filer_window->sort_order;
+	for (i=0; i < G_N_ELEMENTS(sort_names); i++)
+	{
+		if (filer_window->sort_type == sorts[i])
+		{
+			current = i;
+			break;
+		}
+	}
+
+	if (current == -1)
+		next = 0;
+	else if (adjust)
+		next = current - 1;
+	else
+		next = current + 1;
+
+	next_wrapped = next % G_N_ELEMENTS(sorts);
+
+	if (next_wrapped != next)
+		dir = (dir == GTK_SORT_ASCENDING)
+			? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
+
+	display_set_sort_type(filer_window, sorts[next_wrapped], dir);
+ 	tip = g_strconcat(_(sort_names[next_wrapped]), ", ",
+			dir == GTK_SORT_ASCENDING 
+				? _("ascending") : _("descending"),
+			NULL);
+	tooltip_show(tip);
+	g_free(tip);
+}
+
 static void toolbar_details_clicked(GtkWidget *widget,
 				    FilerWindow *filer_window)
 {
@@ -395,6 +466,27 @@ static void toolbar_hidden_clicked(GtkWidget *widget,
 	display_set_hidden(filer_window, !filer_window->show_hidden);
 }
 
+static gboolean invert_cb(ViewIter *iter, gpointer data)
+{
+	return !view_get_selected((ViewIface *) data, iter);
+}
+
+static void toolbar_select_clicked(GtkWidget *widget, FilerWindow *filer_window)
+{
+	GdkEvent	*event;
+
+	event = gtk_get_current_event();
+	if (event->type == GDK_BUTTON_RELEASE)
+	{
+		if (((GdkEventButton *) event)->button == 1)
+			view_select_all(filer_window->view);
+		else
+			view_select_if(filer_window->view, invert_cb,
+				       filer_window->view);
+	}
+	filer_window->temp_item_selected = FALSE;
+}
+
 /* If filer_window is NULL, the toolbar is for the options window */
 static GtkWidget *create_toolbar(FilerWindow *filer_window)
 {
@@ -403,6 +495,7 @@ static GtkWidget *create_toolbar(FilerWindow *filer_window)
 	int		i;
 
 	bar = gtk_toolbar_new();
+
 	if (filer_window)
 		gtk_widget_set_size_request(bar, 100, -1);
 
@@ -503,6 +596,7 @@ static GtkWidget *add_button(GtkWidget *bar, Tool *tool,
 			   icon_widget,
 			   NULL, NULL,	/* CB, userdata */
 			   GTK_TOOLBAR(bar)->num_children);
+	GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
 	
 	if (o_toolbar.int_value == TOOLBAR_HORIZONTAL)
 	{
@@ -618,7 +712,7 @@ static void tally_items(gpointer key, gpointer value, gpointer data)
 	guchar *leafname = (guchar *) key;
 	int    *tally = (int *) data;
 
-	if (leafname[0] == '.')
+	if (!filer_match_filter(filer_window_being_counted, leafname))
 		(*tally)++;
 }
 
